/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * File: statistics.c
 * Author: Ingo Schroeder
 * Purpose: statistics evaluation
 * Birth: 14/4/97
 *
 * $Id: statistics.c,v 1.30 2005-12-01 16:19:05 micha Exp $
 */

/* ---------------------------------------------------------------------- */
#include <config.h>

#include <stdio.h>
#include <blah.h>
#include "cdg.h"
#include "hook.h"
#include "input.h"
#include "lexemgraph.h"
#include "constraintnet.h"
#include "eval.h"
#include "functions.h"
#include "predicates.h"
#include "statistics.h"
#include "set.h"

/* ---------------------------------------------------------------------- */
/* -- TYPE DEFINITIONS -------------------------------------------------- */
/* -- FORWARD DECLARATIONS ---------------------------------------------- */
/* -- VARIABLES --------------------------------------------------------- */
Boolean statUseStatisticsFlag = FALSE;

/* ---------------------------------------------------------------------- */

/* ----------------------------------------------------------------------
 * computes 
 *
 *                                    L(l_i \| c_i. c_{m_i})
 * L(c_i, l_i, c_{m_i} \| \Omega ) = -------------------------
 *                                    \sum L( l \| c_i, c_k )
 */
Number statUnary(LevelValue lv, ConstraintNet net)
{
  List l, p;
  Number sum, para;
  List regents;
  String label;
  int i;
  LexemGraph lg = net->lexemgraph;

  /*
   * inefficient, complicated, but quick (and dirty)
   * Some parameters (or at least) the denominator could be cached.
   */

  for ( p = inputCurrentGrammar->parameters; p != NULL; p = p->next ) {
    net->statUnary++;
    if ( evalUnaryConstraint( (Constraint)p->item, net, NULL, lv) )
      break;
  }

  /* no parameter found, so what shall we do? */
  if ( p == NULL ) {
    if ( hkVerbosity & CDG_INFO ) {
      cdgPrintf(CDG_INFO, "INFO: no parameter found for " );
      lvPrint(CDG_INFO, lv, NULL, 0);
      cdgPrintf(CDG_INFO, "\n" );
    }
    return( 0.0 );
  }

  para = ((Constraint)p->item)->penalty;

  /* save level value */
  regents = lv->regents;
  label = lv->label;

  sum = 0.0;
  for ( l = lv->level->labels; l != NULL; l = l->next ) {
    lv->label = (String)l->item;
    /* 
     * This is just an approximation. Not all lexemnodes from the lexem 
     * graph are suitable. Use of distance??
     */
    for ( i = 0; i < vectorSize( lg->nodes ); i++ ) {
      lv->regents = listPrependElement(NULL, vectorElement( lg->nodes, i ));
      if ( vectorElement( lg->nodes, i ) == lv->dependents->item
	   ||
	   !lgCompatibleSets( lg, lv->dependents, lv->regents )
	   )
	continue;
      for ( p = inputCurrentGrammar->parameters; p != NULL; p = p->next ) {
	net->statUnary++;
	if ( evalUnaryConstraint( (Constraint)p->item, net, NULL, lv ) )
	  break;
      }
      if ( p != NULL ) 
	sum += ((Constraint)p->item)->penalty;
    }

    /* NIL as modifiee */
    lv->regents = NULL;
    for ( p = inputCurrentGrammar->parameters; p != NULL; p = p->next ) {
      net->statUnary++;
      if ( evalUnaryConstraint( (Constraint)p->item, net, NULL, lv ) )
	break;
    }
    if ( p != NULL ) 
      sum += ((Constraint)p->item)->penalty;
  }
  /* restore level value */
  lv->label = label;
  lv->regents = regents;

  if ( sum < para ) {
    fprintf(stderr, "ERROR: invalid fraction\n" );
    abort();
  }

  if ( hkVerbosity & CDG_INFO ) {
    cdgPrintf(CDG_INFO, "INFO: parameter %f for ", para / sum );
    lvPrint(CDG_INFO, lv, NULL, 0);
    cdgPrintf(CDG_INFO, "\n" );
  }

  return( para / sum );
}

/* ----------------------------------------------------------------------
 *
 */
void statInitialize()
{
  setRegister("statistics", SET_BOOL, &statUseStatisticsFlag, NULL, NULL, NULL, NULL);
}

/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */


