/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Author: Horia F. Pop
 * Purpose: demo for libcdg/vars.c
 * Date: February 11, 2000
 *
 * $Id: vartest.c,v 1.3 2005-12-01 16:20:21 micha Exp $ 
 */

#include <assert.h>
#include "config.h"
#include "cdg.h"
#include "hook.h"
#include "variables.h"

//#define VERBOSE
Boolean varFunction(int i)
{
  switch (i) {
    case 10:
    case 20:
    case 30:
      return TRUE;
    default:
      return FALSE;
  }
}

void varDemo()
{
  char sc1[] = "val-sc1";
  char sc2[] = "val-sc2";
  String s1 = NULL;
  String s2 = NULL;
  String s3 = NULL;
  int i1 = 10;
  int i2 = 20;
  List l = NULL;
  Boolean bi, bs;

  void varPrintVars(String s) {
#ifdef VERBOSE
    cdgPrintf(CDG_INFO, "%s\n", s);
    cdgPrintf(CDG_INFO, "s1 = `%s', s2 = `%s', s3=`%s'\n", s1, s2, s3);
    cdgPrintf(CDG_INFO, "i1 = %d, i2 = %d\n", i1, i2);
    cdgPrintf(CDG_INFO, "\n");
#endif
  }

  void varPrintBoolean(String s, Boolean b) {
#ifdef VERBOSE
    cdgPrintf(CDG_INFO, "%s ", s);
    if (b) {
      cdgPrintf(CDG_INFO, "success\n");
    } else {
      cdgPrintf(CDG_INFO, "failure\n");
    }
#endif
    assert(b);
  }



  s1 = sc1;
  s2 = sc2;
  s3 = strRegister("val-s3-longer");

  varPrintVars("Initial");

  bi = varRegister("i1", VAR_INT, &i1, varFunction, NULL, NULL);
  bs = varRegister("s1", VAR_STRING, &s1, NULL, NULL, NULL);

  varPrintBoolean("varRegister on i1 returned", bi);
  varPrintBoolean("varRegister on s1 returned", bs);

  varPrintVars("After registering");

  bi = varSet("i1", 50);
  bs = varSet("s1", strRegister("long-string"));
  /* possible calls with string argument: varSet("s1", sc3); varSet("s1",
   * s3); varSet("s1", strRegister("long-string")); varSet("s1", "foo"); */

  varPrintBoolean("varSet on i1 returned", !bi);
  varPrintBoolean("varSet on s1 returned", bs);

  varPrintVars("After set");

  bi = varGet("i1", &i2);
  bs = varGet("s1", &s2);

  varPrintBoolean("varGet on i1 returned", bi);
  varPrintBoolean("varGet on s1 returned", bs);

  varPrintVars("After get");

  s1 = strRegister("val-modif");
  i1 = 30;

  varPrintVars("After local change");

#ifdef VERBOSE
  cdgPrintf(CDG_INFO, "Contents of variables list\n");
#endif
  for (l = varReturnListOfNames(); l; l = l->next) {
    String s = l->item;

    switch (varReturnType(s)) {
      case VAR_INT:
#ifdef VERBOSE
	cdgPrintf(CDG_INFO, "%s = int\n", s);
#endif
	break;
      case VAR_FLOAT:
#ifdef VERBOSE
	cdgPrintf(CDG_INFO, "%s = float\n", s);
#endif
	break;
      case VAR_BOOL:
#ifdef VERBOSE
	cdgPrintf(CDG_INFO, "%s = Boolean\n", s);
#endif
	break;
      case VAR_STRING:
#ifdef VERBOSE
	cdgPrintf(CDG_INFO, "%s = String\n", s);
#endif
	break;
      default:
#ifdef VERBOSE
	cdgPrintf(CDG_INFO, "%s = ILLEGAL TYPE\n", s);
#endif
	abort();
	break;
    }
  }

#ifdef VERBOSE
  cdgPrintf(CDG_INFO,
	    "\nContents of variables list using varIterator object\n");
#endif
  {
    VarIterator vi = NULL;
    String s = NULL;

    for (varIteratorNew(&vi); (s = varIteratorNextName(&vi));) {
#ifdef VERBOSE
      cdgPrintf(CDG_INFO, " vi == %p\n", vi);
      cdgPrintf(CDG_INFO, "&vi == %p\n", &vi);
#endif
      switch (varReturnType(s)) {
	case VAR_INT:
#ifdef VERBOSE
	  cdgPrintf(CDG_INFO, "%s = int\n", s);
#endif
	  break;
	case VAR_FLOAT:
#ifdef VERBOSE
	  cdgPrintf(CDG_INFO, "%s = float\n", s);
#endif
	  break;
	case VAR_BOOL:
#ifdef VERBOSE
	  cdgPrintf(CDG_INFO, "%s = Boolean\n", s);
#endif
	  break;
	case VAR_STRING:
#ifdef VERBOSE
	  cdgPrintf(CDG_INFO, "%s = String\n", s);
#endif
	  break;
	default:
#ifdef VERBOSE
	  cdgPrintf(CDG_INFO, "%s = ILLEGAL TYPE\n", s);
#endif
	  abort();
	  break;
      }
    }
#ifdef VERBOSE
    cdgPrintf(CDG_INFO, " vi == %p\n", vi);
    cdgPrintf(CDG_INFO, "&vi == %p\n", &vi);
#endif
    varIteratorFree(&vi);
  }

}

int main(void)
{
  cdgInitialize();
  varDemo();
  varFinalize();
  cdgFinalize();
  return 0;
}
