/* 
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 * for (i = 0; i < vectorSize(v); i++)
    printf("%d -> %s\n", i, (String) vectorElement(v, i));

 * Functions of the Vector module tested are
 *
 * vectorNew
 * vectorAddElement
 * vectorRemoveElement
 * vectorRemoveElement
 * vectorInsertElement
 * vectorSetElement
 * vectorSetElements
 * vectorSetAllElements
 * vectorIndexOf
 * vectorCapacity
 * vectorSize
 * vectorIsEmpty
 * vectorContains
 * vectorClone
 * vectorCopy
 * vectorCopy
 *
 * Functions not tested
 *
 * vectorSortWithData
 * doSorting
 */

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <blah.h>
#include <assert.h>

char *a[] = {
  "null", "eins", "zwei", "drei", "vier", "fuenf", "sechs", "sieben",
  "acht", "neun", "zehn", "elf", "zwoelf", "dreizehn", "vierzehn"
};

Boolean less(String a, String b)
{
  if (a == NULL)
    return (FALSE);
  return (b == NULL || strcmp(a, b) < 0);
}

int main(int argc, char **argv)
{
  int i,capa;
  String a1,b1;
  Vector vectorAfterClone;
  Vector vectorAfterCopy = vectorNew(20);
  Vector v;

  blahInitialize();

 /* -----------------------------------------
  * Test for vectorNew
  */

  v = vectorNew(3);
  assert(vectorSize(v)== 0);
  assert(vectorCapacity(v)== 3);

 /* -----------------------------------------
  * Test for vectorAddElement
  */

  for (i = 0; i < 15; i++) {
    assert(vectorSize(v)==i);
    (void)vectorAddElement(v, a[i]);
  }
  assert(vectorSize(v)==15);
  assert(vectorIsEmpty(v)==FALSE);

  /* -----------------------------------------
   * Test for vectorElement
   */

  for (i = 0; i < vectorSize(v); i++)
    assert((String)vectorElement(v, i)==a[i]);

  /* -----------------------------------------
   * Test for vectorRemoveElement
   */

 assert((String)vectorElement(v, 12)==a[12]);
 vectorRemoveElementAt(v,12);
 assert((String)vectorElement(v, 12)==a[13]);
 assert(vectorSize(v)==14);


  /* -----------------------------------------
   * Test for vectorSetElement,vectorContains
   */

 assert(vectorContains(v,"acht")==TRUE);
 assert((String)vectorElement(v, 8)==a[8]);
 vectorSetElement(v,"neue acht",8);
 assert(vectorContains(v,"acht")==FALSE);
 assert(vectorContains(v,"neue acht")==TRUE);

  /* -----------------------------------------
   * Test for vectorSort
   */

 vectorSort(v, less);
 for (i = 0; i < 12; i++)
 {
 a1=((String)vectorElement(v,i));
 b1=((String)vectorElement(v, i+1));
 assert(strcmp(a1,b1)<0);
 }

 /* -----------------------------------------
  * Test for vectorIndexOf
  */

  assert((String)vectorElement(v, 10)==a[4]);
  assert(vectorIndexOf(v,a[4],0)==10);
  
 /* -----------------------------------------
  * Test for vectorIsEmpty
  */
  
  for (i = 0; i < 14; i++)
  {
  a1=((String)vectorElement(v,i));
  assert(strcmp(a1,"")>0);
  }
  assert(vectorIsEmpty(v)==FALSE);
 

  /* -----------------------------------------
   * Test for vectorClone
   */

  vectorAfterClone=vectorClone(v);
  for (i = 0; i < 13; i++)
  {
  a1=((String)vectorElement(v,i));
  b1=((String)vectorElement(vectorAfterClone,i));
  assert(strcmp(a1,b1)==0);
  }

  /* -----------------------------------------
   * Test for vectorSetElements
   */

   vectorSetElements(vectorAfterClone,a[0],14,20);
   for (i = 14; i < 19; i++)
   {
   a1=((String)vectorElement(vectorAfterClone,i));
   assert(strcmp(a1,(String)a[0])==0);
   }

  /* -----------------------------------------
   * Test for vectorSetAllElements
   */

   vectorSetAllElements(vectorAfterClone,a[0]);
   for (i = 0; i < 19; i++)
   {
   a1=((String)vectorElement(vectorAfterClone,i));
   assert(strcmp(a1,(String)a[0])==0);
   }

   vectorDelete(vectorAfterClone);

   /* -----------------------------------------
   * Test for vectorRemoveElement,vectorInserElementAt
   */
   
   assert((String)vectorElement(v, 13)==a[2]);
   assert(vectorContains(v,"zwei")==TRUE);

   vectorRemoveElement(v,a[2]);
   assert(vectorContains(v,"zwei")==FALSE);
   
   vectorInsertElement(v,a[2],13);
   assert(vectorContains(v,"zwei")==TRUE);
   
  
  /* -----------------------------------------
   * Test for resizeVector(doubles the capacity)
   */
   
  capa=vectorCapacity(v);
  vectorSetElement(v, "horst", capa+1); /* triggers resizeVector */
  assert(vectorCapacity(v)==2*capa);

  /* -----------------------------------------
   * Test for vectorCopy
   */

  vectorAfterCopy=vectorCopy(vectorAfterCopy,v);

  for (i = 0; i < 13; i++)
  {
  a1=((String)vectorElement(v,i));
  b1=((String)vectorElement(vectorAfterCopy,i));
  assert(strcmp(a1,b1)==0);
  }
  
  vectorDelete(vectorAfterCopy);
  vectorDelete(v);
  
  blahFinalize();
  return 0;
}
