#!/usr/bin/env perl
# cdg/utils/cda2png.pl
use warnings;
use strict;

BEGIN {
  use File::Basename;
  unshift @INC, dirname(readlink($0) || $0); 
  require 'cdgsetup.pl';
}
use CDG;

use PostScript::Simple;

our $mainlevel = 'SYN';


our @words;
our @edges;
our @labels;
our @children;
our @x;
our @level;

sub y_coord {
  my $n = shift;
  40 + 100*$level[$n];
}

sub height {
  my $x = shift;
  my $result = 0;
  while($x) {
    $x = $edges[$x];
    $result++;
    if($result > $#edges) {
      warn "cyclical tree!\n";
      return $#edges;
    }
  }
  return $result;
}


sub siblingCheck {
  my $down = shift;
  my $up = $edges[$down];
  my @l = $down<$up ? ($down..$up-1) : ($up-1..$down);
  for(@l) {
    return 0 if ! defined $edges[$_];
    return 0 if $edges[$_] != $up;
  }
  return 1;
}

my $cursor;
my $filename = shift;
die "Usage: $0 foo.cda\n" unless -f $filename;

# Read .cda
commandEval "load $filename";

my $annos = InputStruct_annotations_get($CDG::inputCurrentGrammar);
for(my $l = hashListOfKeys $annos;
    $l;
    $l = ListStruct_next_get $l) {
  my $ae = hashGet($annos, ListStruct_item_get $l);
  my $id = AnnoEntryStruct_id_get $ae;
  print "$id\n";

  @words = @edges = @labels = @children = @x = @level = ();
  $cursor = 0;

  for(my $m = AnnoEntryStruct_annos_get $ae;
    $m;
    $m = ListStruct_next_get $m) {
    my $a = ListStruct_item_get $m;
    my $t = AnnotationStruct_to_get $a;
    my $w = AnnotationStruct_word_get $a;
    my $specs = AnnotationStruct_specs_get $a;
    for (my $ss = $specs;
         $ss;
         $ss = ListStruct_next_get($ss)) {
      my $s = ListStruct_item_get($ss);
      if (SpecificationStruct_type_get($s) eq $CDG::STDep) {
        my $level = SpecificationStruct_kind_get $s;
        if($mainlevel eq $level) {
          $words[$t] = $w;
          $edges[$t] = SpecificationStruct_position_get $s;
          $labels[$t] = SpecificationStruct_name_get $s;
          $cursor += 10*length($words[$t]);
          $x[$t] = $cursor;
          $cursor += 10*length($words[$t]);
          push @{$children[$edges[$t]]}, $t;
        }
      }
    }
  }

  # Layout tree
  my $max = 0;
  for(1..$#edges) {
    my $y = height($_);
    $max = $y if $max < $y;
    #  print STDERR "`$words[$_]': level $y\n";
    $level[$_] = $y;
  }

  for(1..$#edges) {
    if(!$children[$_] && !$edges[$_]) { # isolated node
      $level[$_] = 0
    }

    elsif (!$edges[$_]) { # root node
      $level[$_] = $max
    }

    elsif (!$children[$_]) { # leaf node
      if (siblingCheck($_)) {
        $level[$_] = 0;
      } else {
        $level[$_] = $max - $level[$_];
      }
    }

    else { # inner node
      $level[$_] = $max - $level[$_];
    }
  }
  $level[0] = 0;


  # Create postscript graphic
  my $scale = 0.8;
  my $p = new PostScript::Simple
  ( units => "bp",
    xsize => $scale * ($cursor+20),
    ysize => $scale * 100 * (1+$max),
    colour => 1,
    reencode => 'ISOLatin1Encoding',
    eps => 1);
  $p->setfont('Helvetica-Bold-iso', 16);

  for (1..$#edges) {
    my $x = $x[$_] - 5*length($words[$_]);
    $p->text($scale * $x,$scale * 10,$words[$_]);
    $p->circle({filled=>1}, $scale * $x[$_],$scale * y_coord($_),4);
  }

  for (1..$#edges) {
    my $g = $edges[$_];
    my $x1 = $x[$_];
    my $y1 = y_coord($_);
    my $x2 = $edges[$_] ? $x[$edges[$_]] : $x[$_];
    my $y2 = y_coord($edges[$_]);

    # root edge
    if ($y2 < $y1) {
      $y2 = $y1 + 40;
    }

    $p->line($scale*$x1, $scale*$y1,
             $scale*$x2, $scale*$y2);

    $p->text($scale* (10 + $x1+0.5*($x2-$x1)),
             $scale* ($y1+0.5*($y2-$y1)),
             $labels[$_]);
  }

  $p->output("$id.ps") or die "Can't write `$id.ps': $!\n";
}
