/*
 * Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
 *  
 * This file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read COPYING in the root of this distribution.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  
 * Author: Ingo Schroeder
 * Purpose: Flex file for CDG pretty printer
 * Birth: 4/8/97
 */

/* ---------------------------------------------------------------------- */
/* definitions */
%{
/* #include "parser.h" */
/* #include "parser.tab.h" */

int lineNo = 1;
int i;

void yywarning( char * );

#define FLEX_BUFFER_LENGTH 1024
%}

%option nounput
%s CTXT_COMMENT CTXT_STRING LCOMMENT

RE_TRUE   true
RE_FALSE  false
RE_LABEL  label
RE_LEVEL  level
RE_IMPL   ->
RE_BIIMPL <->
RE_GTEQ   >=
RE_LSEQ   <=
RE_NEQ    !=
RE_IDB    :=

RE_NUM    [0-9]+|[0-9]*"."[0-9]+|"-"[0-9]+|"-"[0-9]*"."[0-9]+
RE_STR    [a-zA-Z\"][a-zA-Z0-9\"]*

%%
  int commentDepth = 0;
  char stringBuffer[FLEX_BUFFER_LENGTH];
  char *stringPtr = NULL;

{RE_TRUE} { printf( "\\textbf{true}" ); }
{RE_FALSE} { printf( "\\textbf{false}" ); }
{RE_LABEL} { printf( "\\textbf{label}" ); }
{RE_LEVEL} { printf( "\\textbf{level}" ); }
{RE_IMPL} { printf( "$\\rightarrow$" ); }
{RE_BIIMPL} { printf( "$\\leftrightarrow$" ); }
{RE_GTEQ} { printf( "$\\geq$" ); }
{RE_LSEQ} { printf( "$\\leq$" ); }
{RE_NEQ} { printf( "$\\neq$" ); }
{RE_IDB} { printf( "$:=$" ); }

<INITIAL>{RE_NUM} {
  printf( "%4.3f", atof(yytext) );
}

<INITIAL>{RE_STR} { printf( "%s", yytext ); }

<INITIAL><<EOF>> {
  yyterminate(); 
}
<INITIAL>\' { stringPtr = stringBuffer; BEGIN(CTXT_STRING); }
<CTXT_STRING>\' {
  BEGIN(INITIAL);
  *stringPtr = '\0';
  printf( "\\lq{}%s\\rq{}", stringBuffer );
}
<CTXT_STRING>\n {
  lineNo++;
  yywarning("string ended at end of line");
  BEGIN(INITIAL);
  *stringPtr = '\0';
  printf( "\\lq{}%s\\rq{}", stringBuffer );
}

<CTXT_STRING><<EOF>> {
  yywarning("string ended at end of file");
  BEGIN(INITIAL);
  *stringPtr = '\0';
  printf( "\\lq{}%s\\rq{}", stringBuffer );
}

<CTXT_STRING>\\\n { lineNo++;  }
<CTXT_STRING>\\/[^\n] { 
  lineNo++;
  if (strlen( stringBuffer ) < FLEX_BUFFER_LENGTH - 1 ) 
    *stringPtr++ = '\\';
}

<CTXT_STRING>[^\\\n\']+ {
  char *yytextPtr = yytext;

  while ( *yytextPtr && strlen(stringBuffer) < FLEX_BUFFER_LENGTH - 1)
    *stringPtr++ = *yytextPtr++;
}

<INITIAL>"//" {
  printf( "{\\em // " );
  BEGIN(LCOMMENT);
}
<INITIAL>"%" {
  printf( "{\\em // " );
  BEGIN(LCOMMENT);
}
<LCOMMENT>"\n" { 
  printf( "}\\par" );
  BEGIN(INITIAL); 
}
<INITIAL>[ \t]+ { printf( " " ); }
<INITIAL,CTXT_COMMENT>\n { lineNo++; printf( "\\par\n" ); }
<INITIAL,CTXT_COMMENT>"/*" { 
  commentDepth++;
  BEGIN(CTXT_COMMENT); 
  printf( "{\\em /*" );
}
<CTXT_COMMENT><<EOF>> {
  yywarning("comment ended at end of file");
  for ( i = 0; i < commentDepth; i++ )
    printf( "*/}" );
  yyterminate();
}

<CTXT_COMMENT>"*"\n { lineNo++; printf( "*\\\\" );}
<CTXT_COMMENT>"*"+[^\n/]+ { printf( "%s", yytext ); }
<CTXT_COMMENT>"*"+"/" { 
  commentDepth--;
  printf( "*/}" );
  if ( commentDepth == 0 ) BEGIN(INITIAL);
}

"~" { printf( "$\\neg$" ); }
"%" { printf( "\\%%" ); }
"#" { printf( "\\#" ); }
"&" { printf( "$\\wedge$" ); }
"|" { printf( "$\\vee$" ); }
"^" { printf( "$\\uparrow$" ); }
"@" { printf( "$\\downarrow$" ); }
";" { printf( ";\\par\\vspace{5mm}" ); }
"_" { printf( "\\_" ); }
"{" { printf( "\\{" ); }
"}" { printf( "\\}" ); }
"\\" { printf( "$\\backslash$" ); }

. { printf( "%s", yytext ); }

%%
/* ---------------------------------------------------------------------- */
/* user code */

int yywrap() 
{
  return 1;
}

void yywarning( s ) 
char *s;
{
  printf( "WARNING: %s\n", s );
}

int main(int argc, char **argv)
{
  ++argv, --argc;		       /* skip over program name */
  if (argc > 0)
    yyin = fopen(argv[0], "r");
  else
    yyin = stdin;

  printf("\\documentclass[10pt,a4paper,german]{article}\n");
  printf("\\usepackage{a4wide}\n");
  printf("\\usepackage[german]{babel}\n");
  printf("\\usepackage[T1]{fontenc}\n");
  printf("\\begin{document}\n");

  yylex();
  printf("\\end{document}\n");

  return 0;
}
