#!/usr/bin/perl -w
# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

# This script removes the following types of lines from
# a file generated by the preprocessor from a C-header:
# 1. Lines which consist only of a newline.
# 2. Lines like
# '# 1 "/usr/local/include/acl.h" 1 3'
# which confuse makedepend.
# 3. Lines like
# '#pragma ident	"@(#)stdlib.h	1.47	99/11/03 SMI"'
# which are C++ only and not needed anyway.


use strict;

my $filename = $ARGV[0];
my @buffer = ();

open(FILE, "<$filename")
  or
  die "\nCouldn't open file $filename\n";

while(my $line = <FILE>)
  {
    if (!($line =~ m/(\# \d)/) && ($line ne $/) && !($line =~ m/\#pragma/))
      {
	push(@buffer, $line);
      }
  }

close(FILE);

open(FILE, "> $filename")
  or
  die "\nCouldn't open file $filename\n";

print FILE "/* This file was automatically generated */\n";
print FILE @buffer;

close(FILE);
