#!/usr/bin/env perl

#
# (c) Ingo Schrder
#

use GD;
use Getopt::Std;
getopt('nm');
our $opt_m;    # add morpholngical info to the tree
our $opt_n;    # add node numbers to the tree

$sf=gdTinyFont; #gdSmallFont;
$nf=gdMediumBoldFont;
$iws=10;
$levelheight=45;
$nodespace=4;

$maxlevel=0;
$w=$h=0;
$lno=$state=0;
while ($l=<STDIN>) {
  $lno++;
  chomp($l);
  if($l =~ /^#[EB]OS/) {
    $lno--;
  }
  elsif ($l=~m/^\#(\d+)\s+(\S+)\s+--\s+([^	]+)\s+(\d+)/) {
    # #500			NP	--		PD	511
    my $id = $1;
    my $n={};
    $n->{'term'}=0;
    $n->{'node'}=$2;
    $n->{'label'}=$3;
    $n->{'mother'}=$4;
    $n->{'level'}=-1;
    $n->{'lifted'}=0;
    $n->{'xmin'}=99999999999;
    $n->{'xmax'}=-1;
    $n->{'xpos'}=-1;
    $n->{'xc'}=-1;
    $n->{'y'}=-1;
    $n->{'width'}=length($n->{'node'})*$nf->width + 8 + 2;
    push(@ns, $n);
    $no2index{$id}=$#ns;
    $index2no{$#ns} = $id;
  } elsif ($state==0 && $l=~m/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\d+)/) {
    # Label			NN	Neut.Akk.Sg.*	NK	507
    my $n={};
    $n->{'term'}=1;
    $n->{'word'}=$1;
    $n->{'node'}=$2;
    $n->{'morph'}=$3;
    $n->{'label'}=$4;
    $n->{'mother'}=$5;
    $n->{'level'}=0;
    $n->{'y'}=0;
    my $myw=0;
    $myw=&Max($myw, length($n->{'morph'})*$sf->width);
    $myw=&Max($myw, length($n->{'word'})*$nf->width);
    $myw=&Max($myw, length($n->{'node'})*$nf->width);
    $n->{'width'}=$myw;
    $n->{'xpos'}=$w;
    $n->{'xc'}=$w+int($myw/2);
    $w+=$myw+$iws;
    push(@ns, $n);
  } else {
    die "Can't parse line $lno: $l!\n";
  }
}
$w-=$iws;

for (my $i=0; $i<=$#ns; $i++) {
  my $n=$ns[$i];
  $n->{'no'}=$i-1;
  if ($n->{'mother'}==0) {
    $n->{'mother'}=-1;
  } else {
    die "Undefined internal node #$n->{'mother'}!"
      unless defined($n->{'mother'});
    $n->{mother}=$no2index{"$n->{mother}"};
  }
  #printf STDERR "%s %d has mother %d\n", $n->{'node'}, $n->{'no'}, $n->{'mother'};
}

# detect circles
for (@ns) {
  my $i = 0;
  my $n = $_;
  while(-1 != $$n{mother}) {
    $n = $ns[$$n{mother}];
    if($i++ > $#ns) {
      die "Cyclical tree!\n";
    }
  }
}


for (my $i=0; $i<=$#ns; $i++) {
  my $n=$ns[$i];
  &PropagateLevel($n) if $n->{'term'};
}

for (my $l=0; $l<=$maxlevel; $l++) {
 I:
  for (my $i=0; $i<=$#ns; $i++) {
    my $n=$ns[$i];
    if ($n->{'level'}==$l) {
      if ($n->{'mother'}>=0) {
	my $m=$ns[$n->{'mother'}];
	$m->{'xmin'}=&Min($m->{'xmin'}, $n->{'xc'});
	$m->{'xmax'}=&Max($m->{'xmax'}, $n->{'xc'});
	$m->{'xc'}=int(($m->{'xmin'}+$m->{'xmax'})/2);
#	if ($m->{'no'}==34) {
#	  printf STDERR "%s %d has mother [%d %d %d] %s %d\n",
#	    $n->{'node'}, $n->{'no'}, $m->{'xmin'}, $m->{'xc'}, $m->{'xmax'}, $m->{'node'}, $m->{'no'};
#	}
      }
    }
  }
}

for (my $l=0; $l<=$maxlevel; $l++) {
 I:
  for (my $i=0; $i<=$#ns; $i++) {
    my $n=$ns[$i];
    next I if $n->{'term'};
    next I unless $n->{'level'}==$l;
  J:
    for (my $j=$i+1; $j<=$#ns; $j++) {
      my $m=$ns[$j];
      next J if $m->{'term'};
      next J unless $m->{'level'}==$l;
      next J if $n->{'xmax'}<$m->{'xmin'};
      next J if $n->{'xmin'}>$m->{'xmax'};
      if ($n->{'lifted'}>=$m->{'lifted'}) {
	$m->{'lifted'}++;
	$m->{'level'}++;
	&PropagateLevel($m);	
      } else {
	$n->{'lifted'}++;
	$n->{'level'}++;
	&PropagateLevel($n);	
      }
    }
  }
}

#
# Try to move nodes horizontally to avoid clashes with edges.
#
for (my $l=0; $l<=$maxlevel; $l++) {
 I2:
  for (my $i=0; $i<=$#ns; $i++) {
    my $n=$ns[$i];
    next I2 if $n->{'term'};
    next I2 unless $n->{'level'}==$l;

    # printf STDERR "NEWXC %s(%d) %d [%d, %d]\n", $n->{'node'}, $n->{'no'}, $n->{'xc'}, $n->{'xmin'}, $n->{'xmax'};
  DELTA:
    for (my $delta=0;
	 $n->{'xc'}+$delta<$n->{'xmax'} && $n->{'xc'}-$delta>$n->{'xmin'};
	 $delta++) {
      my $newxc=$n->{'xc'}+$delta;
      my $left=$newxc-int($n->{'width'}/2)-$nodespace;
      my $right=$newxc+int($n->{'width'}/2)+10+$nodespace;
      # printf STDERR "NEWXC %s(%d %d) %d -> %d (%d conflicts) %d %d\n",
      # $n->{'node'}, $n->{'no'}, $n->{'width'}, $n->{'xc'}, $newxc, &Conflicts($i, $newxc, $left, $right), $left, $right;
      if (&Conflicts($i, $newxc, $left, $right)==0) {
	# printf STDERR "NEWXC %s %d -> %d\n", $n->{'node'}, $n->{'xc'}, $newxc unless $n->{'xc'}==$newxc;
	$n->{'xc'}=$newxc;
	last DELTA;
      }
      $newxc=$n->{'xc'}-$delta;
      $left=$newxc-int($n->{'width'}/2)-$nodespace;
      $right=$newxc+int($n->{'width'}/2)+10+$nodespace;
      if (&Conflicts($i, $newxc, $left, $right)==0) {
	# printf STDERR "NEWXC %s %d -> %d\n", $n->{'node'}, $n->{'xc'}, $newxc unless $n->{'xc'}==$newxc;
	$n->{'xc'}=$newxc;
	last DELTA;
      }
    }
  }
}

$h=0;
# terminal node (morph, word, no, tag)
$th=$sf->height+2 + $nf->height+3 + $sf->height + $nf->height;
$h+=$th;
$h+=$maxlevel*$levelheight;
# FIXME:
$h+=30;

$im=new GD::Image($w, $h);
$black=$im->colorAllocate(0, 0, 0);
$white=$im->colorAllocate(255, 255, 255);
$red=$im->colorAllocate(255, 0, 0);
$green=$im->colorAllocate(0, 255, 0);
$blue=$im->colorAllocate(0, 0, 128);
$foo=$im->colorAllocate(42, 42, 42);
#$im->transparent($white);
$im->fill(1, 1, $white);

# gdTinyFont, gdSmallFont, gdMediumBoldFont, gdLargeFont, gdGiantFont

for (my $i=0; $i<=$#ns; $i++) {
  my $n=$ns[$i];
  #printf STDERR "%s %d %d %d %d\n", $n->{'node'}, $n->{'no'}, $n->{'level'}, $n->{'xmin'}, $n->{'xmax'};

  if ($n->{'term'}) {
    my $y=0;
    if($opt_m) {
      String($sf, $n->{'xc'}, $y, $n->{'morph'}, $black);
      $y+=2+$sf->height;
    }
    String($nf, $n->{'xc'}, $y, $n->{'word'}, $red);
    $y+=3+$nf->height;
    if($opt_n) {
      String($sf, $n->{'xc'}, $y, $n->{'no'}, $green);
      $y+=$sf->height;
    }
    String($nf, $n->{'xc'}, $y, $n->{'node'}, $black);
    $y+=2+$nf->height;
  }

  if ($n->{'mother'}>=0) {
    my $m=$ns[$n->{'mother'}];
    my $x1=$n->{'xc'};
    my $x2=$m->{'xc'}; #-int($m->{'width'}/2)-8;
    my $y1=$th+$n->{'level'}*$levelheight;
    my $y2=$th+$m->{'level'}*$levelheight;
    Line($x1, $y1, $x1, $y1+15, $black);
    Rectangle($x1-int(length($n->{'label'})*$sf->width/2)-2,
	      $y1+15,
	      $x1+int(length($n->{'label'})*$sf->width/2)+2,
	      $y1+15+4+$sf->height,
	      $black);
    String($sf, $x1+1, $y1+15+2, $n->{'label'}, $black);
    Line($x1, $y1+15+4+$sf->height, $x1, $y2, $black);
    Line($x1, $y2, $x2, $y2, $black);
  }
}

for (my $i=0; $i<=$#ns; $i++) {
  my $n=$ns[$i];
  if (!$n->{'term'}) {
    Arc($n->{'xc'},
	$th+$n->{'level'}*$levelheight,
	length($n->{'node'})*$nf->width+8,
	$nf->height+5,
	0, 360,
	$foo);
    FillToBorder($n->{'xc'},
		 $th+$n->{'level'}*$levelheight,
		 $foo, $blue);
    Arc($n->{'xc'},
	$th+$n->{'level'}*$levelheight,
	length($n->{'node'})*$nf->width+8,
	$nf->height+5,
	0, 360,
	$blue);
    String($nf,
	   $n->{'xc'},
	   $th+$n->{'level'}*$levelheight-int($nf->height/2),
	   $n->{'node'},
	   $white);
    if($opt_n) {
      String($sf,
             $n->{'xc'}+15,
             $th+$n->{'level'}*$levelheight-int($nf->height/2)-$sf->height,
             $index2no{$n->{'no'}},
             $green);
    }
  }
}

print $im->png;

exit 0;

# ------------------------------------------------------------
sub String () {
  my($f, $x, $y, $s, $c)=@_;
  $im->string($f, $x-int((length($s)*$f->width)/2), $h-$y-$f->height, $s, $c);
}

# ------------------------------------------------------------
sub Line () {
  my($x1, $y1, $x2, $y2, $c)=@_;
  $im->line($x1, $h-$y1, $x2, $h-$y2, $c);
}

# ------------------------------------------------------------
sub Rectangle () {
  my($x1, $y1, $x2, $y2, $c)=@_;
  $im->rectangle($x1, $h-$y1, $x2, $h-$y2, $c);
}

# ------------------------------------------------------------
sub Arc () {
  my($xc, $yc, $dx, $dy, $a, $b, $c)=@_;
  $im->arc($xc, $h-$yc, $dx, $dy, $a, $b, $c);
}

# ------------------------------------------------------------
sub FillToBorder () {
  my($x, $y, $bc, $c)=@_;
  $im->fillToBorder($x, $h-$y, $bc, $c);
}

# ------------------------------------------------------------
sub PropagateLevel () {
  my($n)=@_;

  if ($n->{'mother'}<0) { return; }

  my $m=$ns[$n->{'mother'}];
  if ($m->{'level'}<=$n->{'level'}) {
    $m->{'level'}=$n->{'level'}+1;
    $maxlevel=&Max($m->{'level'}, $maxlevel);
    &PropagateLevel($m);
  }
}

# ------------------------------------------------------------
sub Conflicts () {
  my($i, $xc, $left, $right)=@_;
  my $n=$ns[$i];
  my $c=0;

  for (my $j=0; $j<=$#ns; $j++) {
    my $m=$ns[$j];
    my $mm=$ns[$m->{'mother'}];
    if ($m->{'level'}<$n->{'level'} && $mm->{'level'}>=$n->{'level'} && $m->{'xc'}>$left && $m->{'xc'}<$right) {
      $c++;
    }
  }
  return $c;
}

# ------------------------------------------------------------
sub Max () {
  my($a, $b)=@_;
  return $a>$b ? $a : $b;
}

# ------------------------------------------------------------
sub Min () {
  my($a, $b)=@_;
  return $a<$b ? $a : $b;
}
