#!/usr/bin/env perl
# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# split a wsj mrg file into one file per sentence
#
# author: Michael Daum <micha@nats.informatik.uni-hamburg.de>
# initial revision: 2003-09-24

use warnings;
use strict;
use Parse::RecDescent;

#$::RD_HINT = 1;

#------------------------------------------------------------------------------
sub main {
  $| = 1;

my $pennGrammar = <<'EOG';
{
  my $nrSentences = 0;
}

sentences: sentence(s) 
  {
    print "\n";
  }

sentence: '(' sExpr(s) ')'
  {
    $nrSentences++;
    print "creating s$nrSentences.mrg     \r";
    open(SENTENCE, ">s$nrSentences.mrg") or die "cannot create s$nrSentences.mrg";
    print SENTENCE "(" . join(' ', @{$item[2]}) . ")\n";
    close SENTENCE;
  }

sExpr: 
  '(' /[^\s\)]+/ sExpr(s) ')' 
  {
    $return = "$item[1] $item[2] " . join(' ',@{$item[3]}) . " $item[4]";
  } | 
  '(' /[^\s\)]+/ /[^\s\)]+/ ')'  
  {
    $return = join ' ', @item[1..4];
  } |
  <error?> <error>


EOG

  my $parser = Parse::RecDescent->new($pennGrammar) 
    or die "ERROR: bad treebank parser\n";

  # call processSentence for each sentence
  undef $/;
  while (my $data = <>) {
    $parser->sentences(\$data) 
      or die "ERROR: bad input reading the treebank\n";
  }
  $/ = "\n";
}
#------------------------------------------------------------------------------
&main();
