# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## DemoWindow - the Toplevel Window for the DemoMode implementation
##
## \author Bjrn Engelmann (see also AUTHORS and THANKS for more)
## $Id: demo.tcl,v 1.12 2005-12-01 16:22:13 micha Exp $
##-----------------------------------------------------------------------------

class DemoWindow {
	inherit ::itk::Toplevel TreeEditorParent

	constructor {args} {}; ## \type TclList
	destructor { exit };
	
	public method parse {};
	public method handlePartialResult {parse_pointer}; ## \type Parse
	
	# TreeEditorParent inplementation
	public method setTitle {title}; ## \type TclString
	public method getTitle {};
	public method setUndo { state }; ## \type TclNumber
	public method setRedo { state }; ## \type TclNumber
	public method getSnapmode {}

	public method getPaintMode {}
	public method setPaintMode {x}; ## \type TclNumber
        private variable _paint 0

	public method getRedraw {}
	public method setRedraw {x}; ## \type TclNumber
        private variable _redraw 1
	
  public method getEval {};

	private variable _title "";
	private variable _text "dies ist ein ganz , ganz langer und verdammt komplizierter Satz .";
	private variable _running 0;
	
	public method parse_sub { input }; ## \type TclString
	
	private variable _status "null";
	public method status {args}; ## \type TclList
}

body DemoWindow::constructor {args} {

	if { [catch {	

	global .cdgmain

	wm withdraw $itk_interior
		
	setHookCmd partialresult "$this handlePartialResult"
	
	itk_component add topframe {
		frame $itk_interior.topframe \
	} {}
	
	
	itk_component add sentence {
		iwidgets::entryfield $itk_interior.topframe.sentence \
		-labeltext "Input :" \
		-command [code $this parse] \
		-textvariable [scope _text] \
	} {}
	
	itk_component add parse_button {
		button $itk_interior.topframe.parse_button \
		-text "Start" \
		-command [code $this parse] \
	} { }
	
	
	itk_component add status_button {
		button $itk_interior.topframe.status_button \
		-text "Status" \
		-command [code $this status show] \
		-height 1
	} { }

	itk_component add busy {
		CdgBusy $itk_interior.topframe.busy \
	} { }

	# make this busy Widget the one all the status reports go to
	.cdgmain setBusy $itk_component(busy)
	
	# the help at the bottom
	itk_component add help {
		CdgHelp $itk_interior.help
	} {}
	
	# make this help Widget the one all the status reports go to
	.cdgmain setHelp $itk_component(help)
	
	pack $itk_component(busy) -side right -padx 5
	pack $itk_component(status_button) -side right -padx 0 -pady 3
	pack $itk_component(parse_button) -side right -padx 2
	pack $itk_component(sentence) -side left -fill x -expand 1 -padx 5
	
	# Tree Editor
	itk_component add treeeditor {
		TreeEditor $itk_interior.treeeditor $this
	} {
	}

	pack $itk_component(topframe) -side top -fill x
	pack $itk_component(treeeditor) -fill both -side top -expand 1
	pack $itk_component(help) -side bottom -fill x

	configure -title "Xcdg - Demo"
	
	wm geometry $itk_interior [wm geometry .]
	wm deiconify $itk_interior

	set te $itk_component(treeeditor)
	$te addParse "NULL"
	
	} e] } {
		puts $e
	}
	
	eval itk_initialize $args
}

body DemoWindow::parse {} {

	if { $_running == 1 } {
		set $_running 0
		$itk_component(busy) interrupt
		return
	}
	
	set _running 1
	$itk_component(parse_button) configure -text "Stop"
	
	$itk_component(treeeditor) clear
	set input [$itk_component(sentence) get]
	
	parse_sub $input
	
	set _running 0
	$itk_component(parse_button) configure -text "Start"

}

body DemoWindow::parse_sub { input } {
	
#	puts $input
	
	set wordgraph [::cmd::Inputwordgraph $input]
	
#	puts $wordgraph
	
	set network [::cmd::Newnet $wordgraph]
	
	if { $network == "" } {   .cdgmain confirm "sorry.. could not build a network."; return }
#	puts $network
#	puts [.cdgmain networks getCData $network]
	
	::cmd::Frobbing $network method=combined pressure=0.0 batch=yes
}

body DemoWindow::handlePartialResult {parse_pointer} {
  global errorInfo
  if {[catch {

    set origin "[ParseStruct_searchStrategy_get $parse_pointer] [ParseStruct_latticeId_get $parse_pointer]"
    
    # Parses explicitly requested by saying `^' in frobbing 
    # are displayed no matter what.
    set override [regexp {^manual frobbing} $origin]
    # when doing an IC run, the parses handed over by the frobbing
    # stage should not be displayed after all, because they already
    # appear in the IC window.

    if {$origin == "frobbing interactive"} {
      return
    }
      
	# set parse $allparses([addParse $parse_pointer])
	set parse [Parse "::#auto"]
	$parse init $parse_pointer
	$parse register

	# only show parses in softland
	set score [$parse getScore]

	set treeedit $itk_component(treeeditor)
	$treeedit addParse $parse
	$treeedit view end

   } msg]}  {
     if {$msg != ""} {
       puts "ERROR: $errorInfo"
     }
   }
}

body DemoWindow::setTitle { title } {
	set _title $title
}

body DemoWindow::getTitle {} {
	return $_title
}

body DemoWindow::setUndo { state } {
# 	if { $state == 1 } {
# 		$itk_component(editmenu) entryconfigure 0 -state normal
# 	} else {
# 		$itk_component(editmenu) entryconfigure 0 -state disabled
# 	}
}

body DemoWindow::setRedo { state } {
# 	if { $state == 1 } {
# 		$itk_component(editmenu) entryconfigure 1 -state normal
# 	} else {
# 		$itk_component(editmenu) entryconfigure 1 -state disabled
# 	}
}

## ----------------------------------------------------------------------
## forward
## ----------------------------------------------------------------------
body DemoWindow::getSnapmode {} {
  # default snap mode
  return "nearest"
}

body DemoWindow::getPaintMode {} {
  return $_paint
}
body DemoWindow::setPaintMode {x} {
  set _paint $x
  $itk_component(treeeditor) paint
}
body DemoWindow::getRedraw {} {
  return $_redraw
}
body DemoWindow::setRedraw {x} {
  set _redraw $x
  $itk_component(treeeditor) redraw
}


body DemoWindow::status {args} {
	
	if {![winfo exists $_status]} {
		if ([catch {
			set _status [DemoStatus .statuswindow]
		} msg]) {
			puts "ERROR: $msg";
		}
	}
	
	eval $_status $args;
}

# ----------------------------------------------------------------------
# Evaluation is always wanted in demo mode.
# ----------------------------------------------------------------------
body DemoWindow::getEval {} {
  return 1
}
