# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## \file error.tcl 
## Overwrite the tcl error suite to deactivated the CdgBusy dialog
## in case of an error. 
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
##
## $Id: error.tcl,v 1.8 2005-12-01 16:22:13 micha Exp $
## ----------------------------------------------------------------------------


## global flags used in catch(), error() and bgerror()
set _errorFlag 0

## global flags used in catch(), error() and bgerror()
set _catchFlag 0

## \comment
## force the autoload of the original bgerror function before we overwrite it
if {[auto_load bgerror]} {
   rename bgerror _bgerror
}
rename catch _catch
rename error _error
## \endcomment


## ----------------------------------------------------------------------------
## overwrite the system bgerror command.
## This function simply flags the error in the global variable \c _errorFlag = 1
## when it is encountered, deactivates a CdgBusy dialog and the delegates the
## rest of the call to the origninal bgerror having been renamed to \c _bgerror.
proc bgerror {args} {;## \type TclList
   global _errorFlag

   set _errorFlag 1
   .cdgmain busy abort
   eval _bgerror $args
}

## ----------------------------------------------------------------------------
## overwrite the system cache command.
## This function basically sets the global variable \c catchFlag = 1 before it
## then calls the system cache command (which was renamed to \c catch). After
## having done that it sets \c catchFlag = 0 again. So we track the part of
## call that is being catched, the effect of which is that we want to deactivate
## the CdgBusy dialog only once in error().
proc catch {script {varName ""}} {;## \type TclCommand, TclString
   global _catchFlag
   if {$varName != ""} {
      upvar $varName var
   } else {
      set var ""
   }
   set _catchFlag 1
   set result 0
   set result [uplevel _catch [list $script] $varName]
   set _catchFlag 0
   
   return $result
}


## ----------------------------------------------------------------------------
## overwrite the system error command.
## This flag safely deactivates the CdgBusy dialog, i.e. when the \c _catchFlag is not
## set. If so it also flags the error having taken place in \c _errorFlag = 1.
proc error {message {info ""} {code ""}} {; ## \type TclString, TclString, TclCommand
   global _errorFlag errorInfo _catchFlag

   if {$_catchFlag == 1} {
      return -code error $message 
   }

   set _errorFlag 1
   .cdgmain busy abort
   ::cmd::Puts "ERROR: $message"

   return -code error $message
}


