# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## CdgMain - The root of the XCDG application.
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## $Id: main.tcl,v 1.66 2006-01-11 16:14:07 micha Exp $
## ----------------------------------------------------------------------------
class CdgMain {
  inherit ::itk::Widget

  itk_option define -grammarpath grammarPath GrammarPath ""
  itk_option define -editor editor Editor "gnuclient %f"

  # public methods
  public method networks {args}; ## \type TclList
  public method constraints {args}; ## \type TclList
  public method lexemes {args}; ## \type TclList
  public method hierarchies {args}; ## \type TclList
  public method wordgraphs {args}; ## \type TclList
  public method printdialog {args}; ## \type TclList
  public method parses {args}; ## \type TclList
  public method levels {args}; ## \type TclList
  public method help {args}; ## \type TclList
  public method menu {args}; ## \type TclList
  public method shell {args}; ## \type TclList
  public method busy {args}; ## \type TclList
  public method files {args}; ## \type TclList
  public method question {text {master "."}}; ## \type TclString, TclCommand
  public method confirm {text {master "."}}; ## \type TclString, TclCommand
  public method not_yet  {{master "."}}; ## \type TclCommand
  public method prefs {args}; ## \type TclList
  public method balloon {args}; ## \type TclList
  public method tabno {args}; ## \type TclList
  public method info {args}; ## \type TclList

  ## Should we confirm exit from the program?
  public variable confirmExit 1

  ## flag to show a tcl error trace or not.
  public variable showTrace 0

  ## does `foo:3' mean the tree named `foo:3', or the edge 3 from tree `foo'?
  public variable colonInTreeNames 0
    
  # GUI Mode
  private variable _mode "normal";
  private variable _busy;
  private variable _help;
  private variable _info "null";
  public method mode {args}; ## \type TclList
  public method setBusy {args}; ## \type TclList
  public method setHelp {args}; ## \type TclList 
  public method setShell {args}; ## \type TclList

  constructor {args} {}; ## \type TclList
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body CdgMain::constructor {args} {
    global cnEdgesFlag \
	nsNormalization \
	nsSearchModifiesNetFlag \
	cnShowDeletedFlag \
	evalSloppySubsumesWarnings \
	statUseStatisticsFlag \
        st \

    # the status frame
    itk_component add statusFrame {
      frame $itk_interior.statusFrame
    } {}
    # the busy button
    itk_component add busy {
	CdgBusy $itk_component(statusFrame).busy 
    } {
      keep -foreground -background -cursor
    }
    set _busy $itk_component(busy)

# these kind of statements update the startup-screen's progress-bar:
$st step 6

    # the question-dialog
    itk_component add question {
	iwidgets::messagedialog $itk_interior.question\
	    -bitmap question\
	    -title "Xcdg: Question"\
	    -modality application 
    } {}
    $itk_component(question) default Cancel
    $itk_component(question) buttonconfigure OK  \

$st step 8

    # the confirm-dialog
    itk_component add confirm {
	iwidgets::messagedialog $itk_interior.confirm\
	    -bitmap error\
	    -title "Xcdg: Confirm"\
	    -modality application 
    } {}
    $itk_component(confirm) default Cancel
    $itk_component(confirm) hide Cancel
    $itk_component(confirm) buttonconfigure OK  \
	-text "Confirm"

$st step 10

    # the print-dialog
    itk_component add printdialog {
	iwidgets::canvasprintdialog $itk_interior.printdialog \
	    -modality application
    } {}

$st step 22

    # the help at the bottom
    itk_component add help {
	CdgHelp $itk_component(statusFrame).help
    } {}
    
    set _help $itk_component(help)

$st step 23

    # lets have a tooltip balloon
    itk_component add balloon {
      Balloon $itk_interior.balloon
    } {}

$st step 24

    # a paned window
    itk_component add pane {
	iwidgets::panedwindow $itk_interior.pane \
	    -orient horizontal
    } {
	keep -background -cursor
    }

$st step 25

    $itk_component(pane) add top
    $itk_component(pane) add bottom
    $itk_component(pane) fraction 70 30

    set top [$itk_component(pane) childsite top]
    set bottom [$itk_component(pane) childsite bottom]

$st step 26

    # the shell
    itk_component add shell {
	CdgShell $bottom.shell \
	    -hscrollmode dynamic \
	    -wrap word
    } {
	keep -background -cursor
    }
    

    $itk_component(shell) autotag INFO \
	-foreground black \
	-background green3 
    

    $itk_component(shell) autotag WARNING \
	-foreground black \
	-background yellow2 


    $itk_component(shell) autotag ERROR \
	-foreground white \
	-background red3 


    $itk_component(shell) autotag DEBUG \
	-foreground white \
	-background blue3 

    $itk_component(shell) autotag PROFILE \
	-foreground black \
	-background lightblue 

	# create the InfoSystem
	set _info [InfoSystem .info]
	$_info addListener $itk_component(shell)

$st step 31

    # tabnotebook with all different views
    itk_component add tabno {
	iwidgets::tabnotebook $top.tabno \
	    -borderwidth 0 \
    } {
	keep -background -cursor
	keep -angle -bevelamount -equaltabs -raiseselect -tabbackground
	keep -backdrop -tabpos
    }

$st step 35

    # 1. page : the loaded files
    set flpage [$itk_component(tabno) add \
		    -label "Files" \
		    -borderwidth 0 \
		    -relief flat]

    itk_component add files {
	AllFiles $flpage.files
    } {
	keep -background -cursor
    }

$st step 39

    # 2. page : the wordgraphs
    set wgpage [$itk_component(tabno) add \
		    -label "Wordgraphs" \
		    -borderwidth 0 \
		    -relief flat \
                    -command [code .cdgmain wordgraphs init_data] ]


    itk_component add wordgraphs {
	AllWordgraphs $wgpage.wordgraphs
    } {
	keep -background -cursor
    }

$st step 45

    # 3. page : networks
    set netpage [$itk_component(tabno) add \
		     -label "Networks" \
		     -borderwidth 0 \
		     -relief flat ]

    itk_component add networks {
	AllNetworks $netpage.networks
    } {
	keep -background -cursor
    }

$st step 54

    # 4. page : Parses
    set parsepage [$itk_component(tabno) add \
		      -label "Parses" \
		      -borderwidth 0 \
		      -relief flat ]
    itk_component add parses {
	AllParses $parsepage.parse
    } {
	keep -background -cursor
    }



$st step 58

$st step 61

    # 6. page : Levels
    set levelpage [$itk_component(tabno) add \
		       -label "Levels" \
		       -borderwidth 0 \
		       -relief flat \
                       -command [code .cdgmain levels init_data] ]

    itk_component add levels {
	AllLevels $levelpage.levels
    } {}

$st step 65

    # 7. page : Constraints
    set conspage [$itk_component(tabno) add \
		      -label "Constraints" \
		      -borderwidth 0 \
		      -relief flat \
                      -command [code .cdgmain constraints init_data] ] 

    itk_component add constraints {
	AllConstraints $conspage.constraints
    } {}

$st step 68

    # 8. page : lexikon
    set lexikonpage [$itk_component(tabno) add \
			 -label "Lexikon"\
			 -borderwidth 0 \
			 -relief flat \
                         -command [code .cdgmain lexemes init_data] ]

    itk_component add lexemes {
      AllLexemes $lexikonpage.lexemes
    } {}

$st step 72

    # 9. page : hierarchies
    set hierarchypage [$itk_component(tabno) add \
			   -label "Hierarchies"\
			   -borderwidth 0 \
			   -relief flat \
                           -command [code .cdgmain hierarchies init_data] ]
    itk_component add hierarchies {
	AllHierarchies $hierarchypage.hierarchies
    } {}

$st step 77

    # the menu
    itk_component add menu {
	CdgMenu $itk_interior.menu
    } { 
	keep -background -cursor
    }

$st step 79

    # the prefs dialog
    itk_component add prefs {
	CdgPrefs $itk_interior.prefs \
	    -title "Xcdg: Preferences" \
	    -master . \
	    -modality application
    } {
	keep -background -cursor
    }

$st step 83

    # adjust the view
    $itk_component(tabno) view 0

    # pack some stuff
    pack $itk_component(menu)  -fill x -side top
    pack $itk_component(pane) -fill both -side top -expand 1
    pack $itk_component(tabno) -fill both -expand 1

$st step 84

    pack $itk_component(files) -fill both -expand 1
    pack $itk_component(wordgraphs) -fill both -expand 1
    pack $itk_component(networks) -fill both -expand 1

$st step 85

    pack $itk_component(parses) -fill both -expand 1
    pack $itk_component(shell) -fill both -expand 1
    pack $itk_component(statusFrame) -fill x -side left -expand 1 -padx 8 -pady 5
    grid $itk_component(help) -row 0 -column 0 -sticky ew 
    grid $itk_component(busy) -row 0 -column 1 -sticky e -ipadx 5
    grid columnconfig $itk_component(statusFrame) 0 -weight 1


    pack $itk_component(levels) -fill both -expand 1
    pack $itk_component(constraints) -fill both -expand 1

$st step 86

    pack $itk_component(lexemes) -fill both -expand 1
    pack $itk_component(hierarchies) -fill both -expand 1


    eval itk_initialize $args

    # late init
    $itk_component(menu) init_data

$st step 100

}

## -----------------------------------------------------------------------------
## delegate commands to the networks-component
## -----------------------------------------------------------------------------
body CdgMain::networks {args} {
    return [eval $itk_component(networks) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the wordgraphs-component
## -----------------------------------------------------------------------------
body CdgMain::wordgraphs {args} {
    return [eval $itk_component(wordgraphs) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the help-component
## -----------------------------------------------------------------------------
body CdgMain::help {args} {
    return [eval $_help $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the menu-component
## -----------------------------------------------------------------------------
body CdgMain::menu {args} {
    return [eval $itk_component(menu) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the shell-component
## -----------------------------------------------------------------------------
body CdgMain::shell {args} {
  return [eval $itk_component(shell) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the dialog-component
## -----------------------------------------------------------------------------
body CdgMain::busy {args} {
    return [eval $_busy $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the files-component
## -----------------------------------------------------------------------------
body CdgMain::files {args} {
    return [eval $itk_component(files) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the parses-component
## -----------------------------------------------------------------------------
body CdgMain::parses {args} {
    return [eval $itk_component(parses) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the levels component
## -----------------------------------------------------------------------------
body CdgMain::levels {args} {
    return [eval $itk_component(levels) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the constraints component
## -----------------------------------------------------------------------------
body CdgMain::constraints {args} {
    return [eval $itk_component(constraints) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the lexemes component
## -----------------------------------------------------------------------------
body CdgMain::lexemes {args} {
    return [eval $itk_component(lexemes) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the hierarchies component
## -----------------------------------------------------------------------------
body CdgMain::hierarchies {args} {
    return [eval $itk_component(hierarchies) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the printdialog-component
## -----------------------------------------------------------------------------
body CdgMain::printdialog {args} {
    return [eval $itk_component(printdialog) $args]
}

## -----------------------------------------------------------------------------
## launch a confirm dialog.
## -----------------------------------------------------------------------------
body CdgMain::confirm {text {master "."}} {
   set confirm $itk_component(confirm)

   $confirm configure -text $text -master $master
   $confirm center
   return [$confirm activate]
}

## -----------------------------------------------------------------------------
## launch a question dialog.
## -----------------------------------------------------------------------------
body CdgMain::question {text {master "."}} {
   set question $itk_component(question)

   $question configure -text $text -master $master
   $question center
   return [$question activate]
}

## -----------------------------------------------------------------------------
## delegate commands to the balloon-component
## -----------------------------------------------------------------------------
body CdgMain::balloon {args} {
    return [eval $itk_component(balloon) $args]
}

## -----------------------------------------------------------------------------
## delegate commands to the prefs-component
## -----------------------------------------------------------------------------
body CdgMain::prefs {args} {
    return [eval $itk_component(prefs) $args]
}

## ----------------------------------------------------------------------------
## launch the confirm-dialog with the 'Under Construction' Message.
## ----------------------------------------------------------------------------
body CdgMain::not_yet {{master "."}} {
  confirm "Under Construction !!!" $master
}

## ----------------------------------------------------------------------------
## delegate commands to the tabnotebook component.
## ----------------------------------------------------------------------------
body CdgMain::tabno {args} {
  return [eval $itk_component(tabno) $args]
}

## ----------------------------------------------------------------------------
## delegate commands to the InfoSystem component.
## ----------------------------------------------------------------------------
body CdgMain::info {args} {
  return [eval $_info $args]
}

## ----------------------------------------------------------------------------
## option -grammarpath.
## ----------------------------------------------------------------------------
configbody CdgMain::grammarpath {
   global tkPriv
   upvar #0 __tk_filedialog data

   if {[winfo exists __tk_filedialog]} {
      set data(selectPath) $itk_option(-grammarpath)
   }
   cd $itk_option(-grammarpath)
}

## ----------------------------------------------------------------------------
## set mode to args or - if args is ommitted, return mode
## ----------------------------------------------------------------------------
body CdgMain::mode {args} {
   if {[llength $args] != 0} {
       set _mode [lindex $args 0]
   }
   return $_mode;
}

## ----------------------------------------------------------------------------
## set the busy component - it can be utilized using CdgMain::busy then
## ----------------------------------------------------------------------------
body CdgMain::setBusy {args} {
    if {[llength $args] != 0} {
        set _busy [lindex $args 0]
    }
}

## ----------------------------------------------------------------------------
## set the help component - it can be utilized using CdgMain::help then
## ----------------------------------------------------------------------------
body CdgMain::setHelp {args} {
    if {[llength $args] != 0} {
        set _help [lindex $args 0]
    }
}

