# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## NewnetDialog - gui for the newnet CDG command.
##
## \author Michael Daum (see also AUTHORS and THANKS for more)
## $Id: newnet.tcl,v 1.9 2005-12-01 16:22:13 micha Exp $
## ----------------------------------------------------------------------------
class NewnetDialog {
   inherit ::iwidgets::Dialog


   # public methods
   public method activate {args}; ## \type TclList

   constructor {args} {}; ## \type TclList

   # private methods
   private method _ok_action {}
   private method _newnet {}
   
   ## list of wordgraphs for which we create a new constraintnet
   private variable _wgIds {}
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body NewnetDialog::constructor {args} {
   global cnUnaryPruningFraction

   hide Apply
   hide Help
   buttonconfigure OK -command [code $this _ok_action]

   set child [childsite]
   itk_component add selection {
      iwidgets::entryfield $child.selection \
		-labeltext "Wordgraph: "\
		-labelpos w \
		-textvariable [scope _wgIds]
   } {}
   
   itk_component add uprune {
      iwidgets::entryfield $child.uprune \
		-labeltext "Unary Pruning: " \
		-labelpos w\
		-textvariable cnUnaryPruningFraction \
		-validate real
   }
   
   iwidgets::Labeledwidget::alignlabels \
		$itk_component(selection) \
		$itk_component(uprune)

   pack $itk_component(selection) \
	$itk_component(uprune) \
	-side top -expand 1 -fill both -padx 5 -pady 5

   eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## activate the dialog with a given set of wordgraph ids.
## ----------------------------------------------------------------------------
body NewnetDialog::activate {args} {
   eval set _wgIds $args
   center .
   Shell::activate 
}

## ----------------------------------------------------------------------------
## compute the nets
## ----------------------------------------------------------------------------
body NewnetDialog::_ok_action {} {

   deactivate
   _newnet

}

## ----------------------------------------------------------------------------
## called by _ok_action
## ----------------------------------------------------------------------------
body NewnetDialog::_newnet {} {
   set newIds ""
   foreach id $_wgIds { 
      set netId [::cmd::Newnet $id ]
      set net [.cdgmain networks getCData $netId]
      lappend newIds $netId
   }
   eval .cdgmain networks setSelection $newIds
	
   return $newIds
}


