# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## WordgraphTable - display a wordgaph in a table.
##
## \todo this class is so seldomly used and of such a limited usage that we
## might consider to remove it completely.
##
## $Id: wordgraph.tcl,v 1.25 2005-12-01 16:22:14 micha Exp $
## ----------------------------------------------------------------------------
class WordgraphTable {
   inherit ::itk::Toplevel


   # public methods
   public method init_data {id}; ## \type TclString
   public method show {{id ""}}; ## \type TclString
   public method hide {}
   
   constructor {id args} {}; ## \type TclString, TclList
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body WordgraphTable::constructor {id args} {
   wm iconname $itk_component(hull) "details"
   wm transient $itk_interior .
   wm resizable $itk_interior 1 0

   configure -title "Xcdg - Details"

   # the widget components
   itk_component add frame {
      frame $itk_interior.frame -borderwidth 8 -relief flat
   } {}

   itk_component add info {
      MyTable $itk_component(frame).info \
		-rows 3 \
		-cols 2 \
		-titlecols 1\
		-titlerows 0\
		-state disabled \
		-colstretchmode last \
		-vscrollmode none \
		-hscrollmode none
   } {}
   $itk_component(info) tag config title \
		-anchor w 
   
   itk_component add table {
      MyTable $itk_component(frame).table \
		-rows 2 \
		-vscrollmode none \
		-hscrollmode none \
		-colstretchmode last \
		-state disabled
   } {}

   $itk_component(table) tag config centered -anchor c
   $itk_component(table) tag col centered 0 1 2
   $itk_component(table) tag config words -anchor w
   $itk_component(table) tag col words 3
   $itk_component(table) tag config active \
		-relief sunken \
		-foreground black \
		-background gray80

   itk_component add hide {
      button $itk_component(frame).hide\
		-text "Hide" \
		-pady 3 \
		-command [code $this hide]
   } { }

   # register the helpmessages
   .cdgmain help sethelpstr \
      $itk_component(hide) "Hide the Details-Window" 

   # packing
   pack $itk_component(frame) -fill both -expand 1
   pack $itk_component(info) -fill x 
   pack $itk_component(table) -fill both -pady 5 -expand 1
   pack $itk_component(hide) -fill x -anchor s -side bottom -expand 1

   # get the initial data
   init_data $id
   
   eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## get the data from the C layer.
## \param id a lattice id to be loaded into the table.
## ----------------------------------------------------------------------------
body WordgraphTable::init_data {id} {
  set table $itk_component(table)
  set info $itk_component(info)
  $info configure -state normal
  $table configure -state normal

  $table setCell 0,0 "From" \
       	  0,1 "To" \
       	  0,2 "Penalty" \
       	  0,3 "Word"

  set lattice [findLattice $id]
  if {$lattice == "NULL"} {
    return
  }

  $info setCell 0,0 " Id" 0,1 $id \
       	 1,0 " Filename" 1,1 [LatticeStruct_filename_get $lattice] \
       	 3,0 " LineNo" 3,1 [LatticeStruct_lineNo_get $lattice]

  set rmax [$table cget -rows]
  set r 1

  for {set m [LatticeStruct_arcs_get $lattice]}\
      {$m != "NULL"} \
      {set m [ListStruct_next_get $m]} \
  {
      set arc [ListStruct_item_get $m]
      set word [ArcStruct_word_get $arc]
      set from [ArcStruct_from_get $arc]
      set to [ArcStruct_to_get $arc]
      set penalty [ArcStruct_penalty_get $arc]
      $table setCell $r,0 $from $r,1 $to $r,2 $penalty $r,3 $word
      incr r
  }
  while {$r<$rmax} {
    $table setCell $r,0 "" $r,1 "" $r,2 "" $r,3 "" 
    incr r
  }
  set rmax $r

  $info configure -state disabled
  $table configure -state disabled
  $table configure -rows $rmax -cols 4 -titlecols 0
}

## ----------------------------------------------------------------------------
## show the toplevel.
## \param id optional lattice id to initialize the table.
## ----------------------------------------------------------------------------
body WordgraphTable::show {{id ""}} {
   if {$id != ""} {
      init_data $id
   }
   wm deiconify $itk_interior
}

## ----------------------------------------------------------------------------
## withdraw the toplevel
## ----------------------------------------------------------------------------
body WordgraphTable::hide {} {
   wm withdraw $itk_interior
}

