#!/bin/sh
#\
exec tclsh "$0" "$@" 

package require Itcl
namespace import -force itcl::*

# ----------------------------------------------------------------------------
# cdg-server
# ----------------------------------------------------------------------------
class CdgServer {

  # variables ----------------------------------------------------------------
  public variable cdgpBinary "/home/yvonne/cdg/cdgp/cdgp"
  public variable yadadir "/home/yvonne/cdg/yada"
  public variable timeout 500

  public variable process 0
  public variable allProcesses
  public variable mostCurrentCdgProcess ""

  # methods ------------------------------------------------------------------
  constructor {args} {}; ## \type TclList
  
  public method stopEvent {cdgProcess}
  public method handle_NEW {{xmlFile ""} {logFile ""} {cdgp ""} {job ""}}
  public method handle_TELL {{process ""} input_for_cdgp}
  public method talkToCdg {cmd}
  public method quitCdgServer {}
  public method handle_STATUS {}
  public method eventLoop {}
}

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body CdgServer::constructor {args} {
  source $yadadir/Process.tcl
  source $yadadir/CdgProcess.tcl
  eval configure $args
}

## ---------------------------------------------------------------------------
## event if a cdg process stops 
## ---------------------------------------------------------------------------
body CdgServer::stopEvent {cdgProcess} {
  set pid [eval $cdgProcess getPid]
  foreach {internal_pid external_pid} [array get allProcesses] {
    if {$external_pid == $pid} {
      # delete the stopped cdg process from the table
      array unset allProcesses $internal_pid
      if {$internal_pid == $mostCurrentCdgProcess} {
        # not nice, there should be a list or info that most current is deleted: 
        set mostCurrentCdgProcess "" ;
      }
    }
  }
}

## ---------------------------------------------------------------------------
## handle server command NEW: start a cdgp process
## ---------------------------------------------------------------------------
body CdgServer::handle_NEW {{xmlFile ""} {logFile ""} {cdgp ""} {job ""}} {
  if {$cdgp != ""} {
    set cdgpBinary $cdgp
  }

  set cdgProcess [CdgProcess #auto \
    -cdgp $cdgpBinary \
    -initFile "none" \
    -xmlFile $xmlFile \
    -logFile $logFile \
    -showProgress 1 \
    -sync 1 \
    -stopCallback [code $this stopEvent] \
    -job $job \
  ]
  
  # start the cdg-process
  set pid [$cdgProcess start $cdgpBinary]
  if {$pid <= 0} {
    puts "ERROR: cannot start cdgp process '$cdgpBinary'"
    continue
  }

  # save external pid and server internal pid
  regexp {cdgProcess(\d+)} $cdgProcess dummy cdgProcessID
  set mostCurrentCdgProcess $cdgProcessID
  set allProcesses($cdgProcessID) $pid
  
  puts "New cdgp-process started."
  puts "server-internal Pid: $cdgProcessID"
  puts "external Pid:        $pid"

  if {$job != ""} {
    puts "Started job $job with pid $cdgProcessID."
  }

  return $cdgProcessID
}

## ---------------------------------------------------------------------------
## handle server command TELL:
## tell a cdg-process a command
## if specified process or most current process is available
## ---------------------------------------------------------------------------
body CdgServer::handle_TELL {{process ""} input_for_cdgp} {
  puts "process $process, input: $input_for_cdgp"
  if {$process == ""} {
    if {$mostCurrentCdgProcess == ""} {
      puts "no cdg-processes available or most current process not available any longer"
    } else {
      set process $mostCurrentCdgProcess
      talkToCdg $input_for_cdgp
    }
  } else {
    if {[array names allProcesses -exact $process] != ""} {
      talkToCdg $input_for_cdgp
    } else {
      puts "specified process not available"
    }
  }
}

## ---------------------------------------------------------------------------
## handle server command QUIT: quit the cdg-server
## ---------------------------------------------------------------------------
body CdgServer::quitCdgServer {} {
  foreach process [array names allProcesses] {
  if {[catch {"cdgProcess$process" tell "quit"} errMsg]} {
      puts "ERROR: $errMsg"
    }
  }
  array unset allProcesses
}

## ---------------------------------------------------------------------------
## talk to a cdg-process
## ----------------------------------------------------------------------------
body CdgServer::talkToCdg {cmd} {
#watch "talk cmd '$cmd'"
  if {[catch {"cdgProcess$process" tell $cmd} errMsg]} {
    puts "ERROR: $errMsg"
  }
}

## ---------------------------------------------------------------------------
## handle STATUS
## ----------------------------------------------------------------------------
body CdgServer::handle_STATUS {} {
  puts "ID on Server\tID"
  foreach {intern extern} [array get allProcesses] {
    puts "$intern\t\t$extern"
  }
}



## ---------------------------------------------------------------------------
## MAIN
## ---------------------------------------------------------------------------
body CdgServer::eventLoop {} {

    fconfigure stdin -blocking 0

  # read input line permanentely
  while {1} {
    set task [gets stdin]
    if {$timeout < 0} { quitCdgServer; break }

    if {$task == ""} {
       after 1000; incr timeout -1
       update
       continue
    }

    # execute commands of the server
    switch -regexp -- $task {
      ^H$ -
      ^HELP$ {
	puts "available commands:
	H or HELP:		display this help
	NEW:			start a cdgp-process
	TELL \[process\] cmd:	tell the command cmd to an existing cdg process
				process default: most current process
	STATUS:			gives a list over all processes 
	CPU:			CPU load 
	QUIT:			quit the server"
      }
      {^NEW(.*)} {
        if {[regexp ^NEW$ $task]} {
	  handle_NEW
	} else {
	  regexp {NEW(?:\s+<([^<]*)>)(?:\s+<([^<]*)>)(?:\s+<([^<]*)>)(?:\s+<([^<]*)>)} \
	    $task dummy xml log cdgp job; puts "\n\nxml: $xml\nlog: $log\ncdg: $cdgp\njob: $job\n"
	  handle_NEW $xml $log $cdgp $job
	}
      }
      {TELL(?:\s+(\d*))?\s+(.*)} {
        regexp {TELL(?:\s+(\d*))?\s+(.*)} $task dummy process input_for_cdgp
	handle_TELL $process $input_for_cdgp
      }
      ^CPU$ {
	set cpu "/bin/cat /proc/loadavg"
	puts [eval exec $cpu]
      }
      ^STATUS$ {
        handle_STATUS
      }
      ^Q$ -
      ^QUIT$ {
	quitCdgServer; break
      }
      default {
	  puts "no available command: '$task'"
      }
    }
  }
}

# create an object of itself and initiate the event loop
CdgServer cdgserver
cdgserver eventLoop
