# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaDifference - difference between two experiments.
## \ingroup YadaRunnableDocuments
##
## \author Michael Daum
##
## $Id: YadaDifference.tcl,v 1.13 2005-12-20 15:05:35 micha Exp $
## ----------------------------------------------------------------------------
class YadaDifference {
  inherit YadaDocument

  # variables ----------------------------------------------------------------
  public variable betterAndFasterBackground "green1"
  public variable betterAndFasterForeground "black"
  public variable betterBackground "green2"
  public variable betterButSlowerBackground "green3"
  public variable betterButSlowerForeground "black"
  public variable betterForeground "black"
  public variable fasterBackground "lightblue"
  public variable fasterForeground "black"
  public variable harderBackground "deepskyblue3"
  public variable harderForeground "black"
  public variable sameBackground "gray"
  public variable sameForeground "black"
  public variable slowerBackground "yellow"
  public variable slowerForeground "black"
  public variable softerBackground "deepskyblue1"
  public variable softerForeground "black"
  public variable weirdBackground "pink"
  public variable weirdForeground "black"
  public variable worseAndSlowerBackground "red3"
  public variable worseAndSlowerForeground "white"
  public variable worseBackground "red1"
  public variable worseButFasterBackground "orange"
  public variable worseButFasterForeground "black"
  public variable worseForeground "white"

  private variable _totalWordgraphs 0
  private variable _fasterWordgraphs 0
  private variable _betterWordgraphs 0
  private variable _betterAndFasterWordgraphs 0
  private variable _betterButSlowerWordgraphs 0
  private variable _softerWordgraphs 0
  private variable _harderWordgraphs 0
  private variable _sameWordgraphs 0
  private variable _worseButFasterWordgraphs 0
  private variable _worseAndSlowerWordgraphs 0
  private variable _worseWordgraphs 0
  private variable _slowerWordgraphs 0
  private variable _weirdWordgraphs 0

  private variable _totalTime1 0
  private variable _totalTime2 0
  private variable _totalScore1 0
  private variable _totalScore2 0
  private variable _noSolutions1 0
  private variable _totalSolTime1 0
  private variable _totalSoftTime1 0
  private variable _noSolutions2 0
  private variable _totalSolTime2 0
  private variable _totalSoftTime2 0
  private variable _noErrors1 0
  private variable _noErrors2 0
  private variable _totalStructRecall1 0
  private variable _totalLexicalRecall1 0
  private variable _totalLabelledRecall1 0
  private variable _totalStructRecall2 0
  private variable _totalLexicalRecall2 0
  private variable _totalLabelledRecall2 0

  private variable _typeOfColumn ; ## \type TclArray
  private variable _scoreOf ; ## \type TclArray
  private variable _structHitsOf ; ## \type TclArray
  private variable _lexicalHitsOf ; ## \type TclArray
  private variable _labelledHitsOf ; ## \type TclArray
  private variable _triesOf ; ## \type TclArray

  private variable _prevExperimentName1 ""
  private variable _prevExperimentName2 ""

  ## the wordgraph selected by a right mouse click on the table.
  private variable _selectedContextWordgraphs ""

  ## current sorting state.
  ## This variable stores the column number allong which the table is sorted.
  private variable _lastSorting 0

  # methods ------------------------------------------------------------------
  constructor {args} {}; ## \type TclList

  public method activationHandle {}
  public method init {}
  public method selectExperiment {experimentNo {experimentName ""}}; ## \type TclNumber, TclString
  public method clear {}
  public method reload {}
  public method loadWordgraphs {{wordgraphs ""}} ; ## \type TclList
  public method getWindowTitle {}
  public method isModified {}
  public method toDom {}
  public method getFileName {}
  
  public method fromDom {document}; ## \type domDoc
  public method fromDomNode {differenceNode}; ## \type domNode

  private method _judgeRow {row}; ## \type TclNumber
  private method _getJudgeOfRow {row}; ## \type TclNumber
  private method _escapeCommand {}
  private method _returnCommand {}
  private method _leaveTableCommand {}
  private method _computeStatistics {}
  private method _insertData {wordgraph args}; ## \type TclString, TclList

  private method _getWordgraphs {}
  private method _getSelectedWordgraphs {}
  private method _getRowOfWordgraph {wordgraph}; ## \type TclString
  private method _getSelectedExperiments {}
  private method _getTypeOfColumn {colNo} ; ## \type TclNumber

  private method _analyseDocument {document experiment}; ## \type DomDocument, TclExperiment
  private method _computeSpeedup {time1 time2}; ## \type TclNumber, TclNumber
  private method _judgeData {measure1a measure1b time1 solutionTime1 softTime1 measure2a measure2b time2 solutionTime2 softTime2}; ## \type TclNumber, TclNumber, TclNumber, TclNumber, TclNumber, TclNumber, TclNumber, TclNumber, TclNumber, TclNumber
  private method _resetDataOf {wordgraphs}; ## \type TclList
  private method _resortRows {}
  private method _print {message} ; ## \type TclString
  private method _showContextMenu {x y xroot yroot}; ## \type TclNumber, TclNumber, TclNumber, TclNumber
  private method _clearRow {}
  private method _sortRows {colNo args} ;## \type TclNumber, TclList
  private method _viewLogfile {}
  private method _viewXmlfile {}

};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaDifference::constructor {args} {

  # set types of columns
  array set _typeOfColumn {
    0 "dictionary"
    1 "integer"
    2 "real"
    3 "integer"
    4 "integer"
    5 "integer"
    6 "integer"
    7 "real"
    8 "real"
    9 "integer"
    10 "integer"
    11 "integer"
    12 "integer"
    13 "real"
    14 "real"
    15 "real"
    16 "real"
    17 "ascii"
  }

  # talbe
  itk_component add table {
     MyTable $itk_component(childsite).table \
       	-exportselection 1 \
	-selecttitles 1 \
	-rowseparator "\n" \
	-colseparator "\t" \
       	-rows 300 \
       	-cols 19 \
       	-state normal \
       	-cellanchor c \
       	-colstretchmode last \
       	-rowstretchmode none \
       	-selectmode extended \
       	-selecttype row \
       	-titlerows 1 \
       	-titlecols 1 \
	-borderwidth 3 \
	-outerborderwidth 2 \
	-outerrelief sunk \
	-selforeground "" \
	-selbackground "" \
	-font {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} \
        -troughcolor gray \
	-hscrollmode dynamic \
	-rowtagcommand [code $this _judgeRow]
  } { 
     keep -background -cursor 
  }

  # view menu
  itk_component add sortIncMenu {
     menu $itk_component(viewMenu).sortIncMenu 
  } {
     keep -background -cursor
     rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }

  itk_component add sortDecMenu {
     menu $itk_component(viewMenu).sortDecMenu
  } {
     keep -background -cursor
     rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }
  $itk_component(viewMenu) add separator
  $itk_component(viewMenu) add cascade \
	-label "Sort inc. by" \
	-underline 0 \
	-menu $itk_component(sortIncMenu)
  $itk_component(viewMenu) add cascade \
	-label "Sort dec. by" \
	-underline 0 \
	-menu $itk_component(sortDecMenu)

  # edit menu
  $itk_component(editMenu) add command \
       	-label "Clear" \
       	-underline 0 \
       	-command [code $this clear]

  # context menu
  itk_component add contextMenu {
     menu $itk_interior.contextMenu 
  } {
     keep -background -cursor
     rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }
  $itk_component(contextMenu) add command \
       	-label "View Log" \
       	-underline 0 \
       	-command [code $this _viewLogfile]
  $itk_component(contextMenu) add command \
       	-label "View Xml" \
       	-underline 5 \
       	-command [code $this _viewXmlfile]
  $itk_component(contextMenu) add command \
       	-label "Clear" \
       	-underline 0 \
       	-command [code $this _clearRow]

  # toolbar
  $itk_component(toolbar) add FlatButton reloadButton \
    -image [YadaImages::get reload] \
    -balloonstr "reload" \
    -command [code $this reload]

  $itk_component(toolbar) add FlatButton clearButton \
	-image [YadaImages::get erase] \
	-command [code $this clear] \
	-balloonstr "clear"

  # heading space
  itk_component add heading {
    frame $itk_component(childsite).heading
  }

  # experiments
  foreach i {1 2} {

    itk_component add experiment${i}BoxLabel {
      label $itk_component(heading).experiment${i}BoxLabel \
	  -text "Experiment ${i}:" \
	  -anchor w
    }  {}

    itk_component add experiment${i}Box {
      iwidgets::combobox $itk_component(heading).experiment${i}Box \
	  -completion false \
	  -borderwidth 2 \
	  -margin 4 \
	  -selectioncommand [code $this selectExperiment $i]
    }  {}

    itk_component add score${i}Label {
      label $itk_component(heading).score${i}Label \
	-text "Score:" \
	-anchor w \
    } {}

    itk_component add score${i} {
      entry $itk_component(heading).score${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add totalTime${i}Label {
      label $itk_component(heading).totalTime${i}Label \
	-text "Time:" \
	-anchor w 
    } {}

    itk_component add totalTime${i} {
      entry $itk_component(heading).totalTime${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add noSolutions${i}Label {
      label $itk_component(heading).noSolutions${i}Label \
	-text "No.Solutions:" \
	-anchor w 
    } {}

    itk_component add noSolutions${i} {
      entry $itk_component(heading).noSolutions${i} \
	-textvariable [scope _noSolutions${i}] \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add noErrors${i}Label {
      label $itk_component(heading).noErrors${i}Label \
	-text "No.Errors:" \
	-anchor w 
    } {}

    itk_component add noErrors${i} {
      entry $itk_component(heading).noErrors${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add totalSolTime${i}Label {
      label $itk_component(heading).totalSolTime${i}Label \
	-text "Sol.Time:" \
	-anchor w
    } {}

    itk_component add totalSolTime${i} {
      entry $itk_component(heading).totalSolTime${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add totalSoftTime${i}Label {
      label $itk_component(heading).totalSoftTime${i}Label \
	-text "Soft Time:" \
	-anchor w
    } {}

    itk_component add totalSoftTime${i} {
      entry $itk_component(heading).totalSoftTime${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add totalStructRecall${i}Label {
      label $itk_component(heading).totalStructRecall${i}Label \
	-text "Struct.Recall:" \
	-anchor w
    } {}
    
    itk_component add totalStructRecall${i} {
      entry $itk_component(heading).totalStructRecall${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    itk_component add totalLexicalRecall${i}Label {
      label $itk_component(heading).totalLexicalRecall${i}Label \
	-text "Lex.Recall:" \
	-anchor w
    } {}

    itk_component add totalLabelledRecall${i}Label {
      label $itk_component(heading).totalLabelledRecall${i}Label \
	-text "Label.Recall:" \
	-anchor w
    } {}
    
    itk_component add totalLabelledRecall${i} {
      entry $itk_component(heading).totalLabelledRecall${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

    
    itk_component add totalLexicalRecall${i} {
      entry $itk_component(heading).totalLexicalRecall${i} \
	-borderwidth 2 \
	-state disabled \
	-width 20
    } {}

   # bindings
    set entry [$itk_component(experiment${i}Box) component entry]
    bind $entry <Return> [code $this selectExperiment $i]
    bind $entry <Up> +[code $this selectExperiment $i]
    bind $entry <Down> +[code $this selectExperiment $i]
  }

  # bindings
  set hull $itk_component(hull)
  set tableInTable [$itk_component(table) component table]
  bind $hull <Leave> [code $this _leaveTableCommand]
  bind $hull <FocusOut> [code $this _leaveTableCommand]
  bind $tableInTable <KeyPress-Escape> [code "$this _escapeCommand; break"]
  bind $tableInTable <KeyPress-Return> [code "$this _returnCommand; break"]
  bind $tableInTable <Button-3> [code "$this _showContextMenu %x %y %X %Y; break"]

  # separator
  itk_component add separator {
    frame $itk_component(heading).separator \
	-height 2 \
	-borderwidth 2 \
	-relief sunk
  }

  #
  # judge bar
  #
  itk_component add judgeFrame {
    frame $itk_component(heading).judgeFrame
  }

  itk_component add totalWordgraphs {
    iwidgets::entryfield $itk_component(judgeFrame).totalWordgraphs \
      -labeltext "Total:" \
      -textvariable [scope _totalWordgraphs] \
      -labelpos w \
      -borderwidth 2 \
      -state disabled \
      -disabledforeground black \
      -width 5
  } {}

  itk_component add judgeBar {
    YadaJudgeBar $itk_component(judgeFrame).judgeBar \
      -height 15 \
      -borderwidth 2 \
      -relief sunk \
      -betterAndFasterBackground $betterAndFasterBackground \
      -betterBackground $betterBackground \
      -betterButSlowerBackground $betterButSlowerBackground \
      -fasterBackground $fasterBackground \
      -softerBackground $softerBackground \
      -sameBackground $sameBackground \
      -harderBackground $harderBackground \
      -slowerBackground $slowerBackground \
      -worseButFasterBackground $worseButFasterBackground \
      -worseBackground $worseBackground \
      -worseAndSlowerBackground $worseAndSlowerBackground \
      -betterForeground $betterForeground \
      -betterAndFasterForeground $betterAndFasterForeground \
      -betterButSlowerForeground $betterButSlowerForeground \
      -fasterForeground $fasterForeground \
      -softerForeground $softerForeground \
      -sameForeground $sameForeground \
      -harderForeground $harderForeground \
      -slowerForeground $slowerForeground \
      -worseButFasterForeground $worseButFasterForeground \
      -worseAndSlowerForeground $worseAndSlowerForeground \
      -worseForeground $worseForeground \
      -weirdForeground $weirdForeground \
      -weirdBackground $weirdBackground
  } {}

  # packing
  grid columnconfigure $itk_component(heading) 1 -weight 1
  grid columnconfigure $itk_component(heading) 2 -pad 5
  grid columnconfigure $itk_component(heading) 3 -weight 1
  grid columnconfigure $itk_component(heading) 4 -pad 5
  grid columnconfigure $itk_component(heading) 5 -weight 1 
  grid columnconfigure $itk_component(heading) 6 -pad 5
  grid columnconfigure $itk_component(heading) 7 -weight 1
  grid columnconfigure $itk_component(heading) 8 -pad 5
  grid columnconfigure $itk_component(heading) 9 -weight 1
  grid columnconfigure $itk_component(heading) 10 -pad 5
  grid columnconfigure $itk_component(heading) 11 -weight 1
  grid columnconfigure $itk_component(heading) 12 -pad 5
  grid columnconfigure $itk_component(heading) 13 -weight 1
  grid columnconfigure $itk_component(heading) 14 -pad 5

  foreach i {1 2} {
    grid $itk_component(experiment${i}BoxLabel) \
      -row 0 -column [expr ($i - 1) * 8]     -sticky ew -pady 2
    grid $itk_component(experiment${i}Box) \
      -row 0 -column [expr ($i - 1) * 8 + 1]     -sticky ew -pady 2 -columnspan 5 

    grid $itk_component(totalTime${i}Label) \
      -row 1 -column [expr ($i - 1) * 8]     -sticky ew -pady 2
    grid $itk_component(totalTime${i}) \
      -row 1 -column [expr ($i - 1) * 8 + 1]     -sticky ew -pady 2

    grid $itk_component(score${i}Label)\
      -row 1 -column [expr ($i - 1) * 8 + 2] -sticky ew -pady 2
    grid $itk_component(score${i})\
      -row 1 -column [expr ($i - 1) * 8 + 3] -sticky ew -pady 2

    grid $itk_component(totalStructRecall${i}Label) \
      -row 1 -column [expr ($i - 1) * 8 + 4]     -sticky ew -pady 2
    grid $itk_component(totalStructRecall${i}) \
      -row 1 -column [expr ($i - 1) * 8 + 5] -sticky ew -pady 2

    grid $itk_component(totalSolTime${i}Label) \
      -row 2 -column [expr ($i - 1) * 8 ] -sticky ew -pady 2
    grid $itk_component(totalSolTime${i}) \
      -row 2 -column [expr ($i - 1) * 8 + 1] -sticky ew -pady 2

    grid $itk_component(noErrors${i}Label) \
      -row 2 -column [expr ($i - 1) * 8 + 2]     -sticky ew -pady 2
    grid $itk_component(noErrors${i}) \
      -row 2 -column [expr ($i - 1) * 8 + 3] -sticky ew -pady 2

    grid $itk_component(totalLabelledRecall${i}Label) \
      -row 2 -column [expr ($i - 1) * 8 + 4]     -sticky ew -pady 2
    grid $itk_component(totalLabelledRecall${i}) \
      -row 2 -column [expr ($i - 1) * 8 + 5] -sticky ew -pady 2

    grid $itk_component(totalSoftTime${i}Label) \
      -row 3 -column [expr ($i - 1) * 8 ] -sticky ew -pady 2
    grid $itk_component(totalSoftTime${i}) \
      -row 3 -column [expr ($i - 1) * 8 + 1] -sticky ew -pady 2

    grid $itk_component(noSolutions${i}Label) \
      -row 3 -column [expr ($i - 1) * 8 + 2]     -sticky ew -pady 2
    grid $itk_component(noSolutions${i}) \
      -row 3 -column [expr ($i - 1) * 8 + 3] -sticky ew -pady 2

    grid $itk_component(totalLexicalRecall${i}Label) \
      -row 3 -column [expr ($i - 1) * 8 + 4]     -sticky ew -pady 2
    grid $itk_component(totalLexicalRecall${i}) \
      -row 3 -column [expr ($i - 1) * 8 + 5] -sticky ew -pady 2

  }

  pack $itk_component(heading) -fill x -anchor n 
  pack $itk_component(table) -fill both -expand 1 -padx 2 -pady 5

  grid $itk_component(separator)  -sticky ew -row 4 -column 0 -pady 10 -columnspan 14
  grid $itk_component(judgeFrame) -sticky ew -row 7 -column 0 -pady 2 -columnspan 14

  grid columnconfigure $itk_component(judgeFrame) 1 -weight 1

  grid $itk_component(totalWordgraphs) -sticky w -row 0 -column 0 -pady 2
  grid $itk_component(judgeBar)     -sticky ew -row 0 -column 1 -pady 2 

  eval itk_initialize $args
  .main registerDocument $this
}

## ----------------------------------------------------------------------------
## activationHandle
## ----------------------------------------------------------------------------
body YadaDifference::activationHandle {} {
  chain

  set experimentNames [.main getActiveExperiments]

  foreach i {1 2} {

  # get the previous selection
    set box $itk_component(experiment${i}Box)
    set experimentName [$box get]

    # default experiments to choose
    if {$experimentName == ""} {
      set experimentName "<none>"
    }

    set listOfEntries [[$box component list] get 0 end]
    if {$listOfEntries != $experimentNames} {
      $box clear list 
      eval $box insert list end $experimentNames
      $box insert entry 0 "$experimentName"
    }
  }
}

## ----------------------------------------------------------------------------
## _judgeRow
## ----------------------------------------------------------------------------
body YadaDifference::_judgeRow {row} {
  set judge [_getJudgeOfRow $row]
  set wordgraph [$itk_component(table) getCell $row,0]

  if {$judge == ""} {
    if {[expr $row % 2]} {
      return "default"
    } else {
      return "colored"
    }
  }
  
  return $judge
}

## ----------------------------------------------------------------------------
## _getJudge
## ----------------------------------------------------------------------------
body YadaDifference::_getJudgeOfRow {row} {
  return [$itk_component(table) getCell $row,17]
}

## ----------------------------------------------------------------------------
## selectExperiment
## ----------------------------------------------------------------------------
body YadaDifference::selectExperiment {experimentNo {experimentName ""}} {
  set box $itk_component(experiment${experimentNo}Box)
  if {$experimentName == ""} {
    set experimentName [$box get]
    if {$experimentName == ""} {
      set experimentName "<none>"
    }
  }
  set prevExperimentName [subst \$_prevExperimentName${experimentNo}]
  if {$prevExperimentName != $experimentName} {
    set _prevExperimentName${experimentNo} $experimentName
    clear
    $box clear entry
    $box insert entry 0 "$experimentName"
  }
  
  set listOfEntries [[$box component list] get 0 end]
  set index [lsearch $listOfEntries $experimentName]

  if {$index < 0} {
    selectExperiment $experimentNo "<none>"
  } else {
    $box selection set $index $index
  }

  updateWindowTitle
  updateMenus
}

## ----------------------------------------------------------------------------
## _escapeCommand
## ----------------------------------------------------------------------------
body YadaDifference::_escapeCommand {} {
  set table $itk_component(table)
  set lastActiveCell [$table tag cell active]
  scan $lastActiveCell "%d,%d" row col
  
  if {$col == 0} {
    $table selection clear all
  } else {
    $table activate -1,-1
  }
}
## ----------------------------------------------------------------------------
## _returnCommand
## ----------------------------------------------------------------------------
body YadaDifference::_returnCommand {} {
  set table $itk_component(table)
  set lastActiveCell [$table tag cell active]
  scan $lastActiveCell "%d,%d" row col
  set colMax [expr [$table cget -cols] -1]
  if {$col == $colMax} {
    incr row
    set col 1
  } else {
    incr col
  }

  $table activate $row,$col
  $table see $row,$col
}

## ----------------------------------------------------------------------------
## _leaveTableCommand
## ----------------------------------------------------------------------------
body YadaDifference::_leaveTableCommand {} {
  set table $itk_component(table)
  set lastActiveCell [$table tag cell active]
  if {$lastActiveCell == ""} {
    return
  }

  scan $lastActiveCell "%d,%d" row col
  $table activate -1,-1
}

## ----------------------------------------------------------------------------
## init
## ----------------------------------------------------------------------------
body YadaDifference::init {} {
  if {$_isInitialized} {
    return
  }
  chain
  clear

  $itk_component(sortIncMenu) delete 0 end
  $itk_component(sortDecMenu) delete 0 end

  set cmax [$itk_component(table) cget -cols]
  set headings [$itk_component(table) getCell 0,0 0,$cmax]
  set colNo 0
  foreach heading $headings {
    if {$heading != ""} {
      $itk_component(sortIncMenu) add command \
	-label "$heading" \
	-command [code $this _sortRows $colNo -increasing]
      $itk_component(sortDecMenu) add command \
	-label "$heading" \
	-command [code $this _sortRows $colNo -decreasing]
    }
    incr colNo
  }
}

## ----------------------------------------------------------------------------
## clear
## ----------------------------------------------------------------------------
body YadaDifference::clear {} {

  set table $itk_component(table)
  set selection [$table curselection]
  set cmax [$table cget -cols]

  setModified

  # only delete the seleted rows
  if {$selection != ""} {
    set minRow 9999
    set maxRow 0
    foreach index $selection {
      scan $index "%d,%d" row col
      $table erase $row,1 $row,$cmax
      $table clear tags $row,0 $row,$cmax
      set wordgraph [$table getCell $row,0]
      _resetDataOf $wordgraph
    }
  } else {
  
    # reset the whole statistics
    $table clear tags 
    $table delete rows -holddimensions 1 [$table cget -rows]

    # insert the wordgraphs
    set wordgraphs [_getWordgraphs]
    set rmax [llength $wordgraphs]
    if {$rmax < 100} {
      set rmax 100
    } else {
      incr rmax
    }
    $table configure -rows $rmax
    set row 1
    foreach wordgraph $wordgraphs {
      $table setCell $row,0 $wordgraph
      incr row
    }

    _resetDataOf "$wordgraphs"
  }

  .main printStatus ""
  # reset the table 
  set table $itk_component(table)
  $table setCell \
       	0,0 "wordgraph" \
       	0,1 "values" \
       	0,2 "score" \
	0,3 "errors" \
       	0,4 "time" \
       	0,5 "sol.time" \
       	0,6 "soft time" \
	0,7 "lex.recall" \
       	0,8 "score" \
	0,9 "errors" \
       	0,10 "time" \
       	0,11 "sol.time" \
       	0,12 "soft time" \
	0,13 "lex.recall" \
	0,14 "speed up" \
       	0,15 "sol.speed up" \
       	0,16 "soft speed up" \
	0,17 "judge"


  $table tag config default -anchor e
  $table tag config colored -bg gray80 -anchor e
  $table tag config better -bg $betterBackground -fg $betterForeground -anchor e -relief flat
  $table tag config betterAndFaster -bg $betterAndFasterBackground -fg $betterAndFasterForeground -anchor e -relief flat
  $table tag config betterButSlower -bg $betterButSlowerBackground -fg $betterButSlowerForeground -anchor e -relief flat
  $table tag config faster -bg $fasterBackground -fg $fasterForeground -anchor e -relief flat
  $table tag config softer -bg $softerBackground -fg $softerForeground -anchor e -relief flat
  $table tag config harder -bg $harderBackground -fg $harderForeground -anchor e -relief flat
  $table tag config worseButFaster  -bg $worseButFasterBackground  -fg $worseButFasterForeground  -anchor e -relief flat
  $table tag config worseAndSlower  -bg $worseAndSlowerBackground  -fg $worseAndSlowerForeground  -anchor e -relief flat
  $table tag config worse  -bg $worseBackground  -fg $worseForeground  -anchor e -relief flat
  $table tag config slower -bg $slowerBackground -fg $slowerForeground -anchor e -relief flat
  $table tag config same   -bg $sameBackground   -fg $sameForeground   -anchor e -relief flat
  $table tag config weird  -bg $weirdBackground -fg $weirdForeground -anchor e -relief flat
  
  $table tag config sel -fg "" -bg "" -relief raised -bd 1
  $table tag config active -anchor w -fg black -bg gray -relief sunk -bd 1
  $table tag config title -font {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} -bg #a3a3a3 -fg white
  $table tag config runner -fg red3 -bg #a3a3a3
  $table tag config judge -anchor w -bg "" -fg ""
  $table tag raise runner title

  $table width 0 15 1 5

  _computeStatistics

}

## ----------------------------------------------------------------------------
## _computeStatistics
## ----------------------------------------------------------------------------
body YadaDifference::_computeStatistics {} {
  chain

  set table $itk_component(table)
  set rmax [$table cget -rows]
  set cmax [$table cget -cols]

  set _totalWordgraphs 0
  set _fasterWordgraphs 0
  set _betterWordgraphs 0
  set _betterAndFasterWordgraphs 0
  set _betterButSlowerWordgraphs 0
  set _softerWordgraphs 0
  set _harderWordgraphs 0
  set _sameWordgraphs 0
  set _worseWordgraphs 0
  set _worseButFasterWordgraphs 0
  set _worseAndSlowerWordgraphs 0
  set _slowerWordgraphs 0
  set _weirdWordgraphs 0
  set _noErrors1 0
  set _noErrors2 0
  set _noSolutions1 0
  set _noSolutions2 0
  set _totalSolTime1 0
  set _totalSolTime2 0
  set _totalSoftTime1 0
  set _totalSoftTime2 0
  set _totalScore1 0.0
  set _totalScore2 0.0
  set _totalTime1 0
  set _totalTime2 0
  set _totalStructRecall1 0
  set _totalLexicalRecall1 0
  set _totalLabelledRecall1 0
  set _totalStructRecall2 0
  set _totalLexicalRecall2 0
  set _totalLabelledRecall2 0
  set totalTries1 0
  set totalStructHits1 0
  set totalLexicalHits1 0
  set totalLabelledHits1 0
  set totalTries2 0
  set totalStructHits2 0
  set totalLexicalHits2 0
  set totalLabelledHits2 0

  for {set row 1} {$row < $rmax} {incr row} {

    set dataRow [$table getCell $row,0 $row,$cmax]
    set noValues [lindex $dataRow 1]
    if {$noValues == ""} {
      continue
    }

    set wordgraph	[lindex $dataRow 0]
    set noErrors1	[lindex $dataRow 3]
    set time1		[lindex $dataRow 4]
    set solutionTime1	[lindex $dataRow 5]
    set softTime1	[lindex $dataRow 6]
    set noErrors2	[lindex $dataRow 9]
    set time2		[lindex $dataRow 10]
    set solutionTime2	[lindex $dataRow 11]
    set softTime2	[lindex $dataRow 12]
    set speedUp		[lindex $dataRow 14]
    set solSpeedUp	[lindex $dataRow 15]
    set softSpeedUp	[lindex $dataRow 16]
    set judge		[lindex $dataRow 17]

    incr _totalWordgraphs
    incr _noErrors1 $noErrors1
    incr _noErrors2 $noErrors2
    incr _totalTime1 $time1
    incr _totalTime2 $time2
    incr _totalSolTime1 $solutionTime1
    incr _totalSoftTime1 $softTime1
    incr _totalSolTime2 $solutionTime2
    incr _totalSoftTime2 $softTime2

    set _totalScore1 [expr $_totalScore1 + $_scoreOf($wordgraph,1)]
    if {$_scoreOf($wordgraph,1) > 0.0} {
      incr _noSolutions1
    }
    set _totalScore2 [expr $_totalScore2 + $_scoreOf($wordgraph,2)]
    if {$_scoreOf($wordgraph,2) > 0.0} {
      incr _noSolutions2
    }

    incr totalTries1 $_triesOf($wordgraph,1)
    incr totalStructHits1 $_structHitsOf($wordgraph,1)
    incr totalLexicalHits1 $_lexicalHitsOf($wordgraph,1)
    incr totalLabelledHits1 $_labelledHitsOf($wordgraph,1)
    incr totalTries2 $_triesOf($wordgraph,2)
    incr totalStructHits2 $_structHitsOf($wordgraph,2)
    incr totalLexicalHits2 $_lexicalHitsOf($wordgraph,2)
    incr totalLabelledHits2 $_labelledHitsOf($wordgraph,2)

    if {$judge != ""} {
      incr _${judge}Wordgraphs
    }
  }

  if {$totalTries1 > 0} {
    set _totalStructRecall1 [expr $totalStructHits1 * 100.0 / $totalTries1  ]
    set _totalLexicalRecall1 [expr $totalLexicalHits1 * 100.0 / $totalTries1  ]
    set _totalLabelledRecall1 [expr $totalLabelledHits1 * 100.0 / $totalTries1  ]
  }
  if {$totalTries2 > 0} {
    set _totalStructRecall2 [expr $totalStructHits2 * 100.0 / $totalTries2  ]
    set _totalLexicalRecall2 [expr $totalLexicalHits2 * 100.0 / $totalTries2  ]
    set _totalLabelledRecall2 [expr $totalLabelledHits2 * 100.0 / $totalTries2  ]
  }

  # show statistics
  foreach i {1 2} {
    set noSolutions [subst \$_noSolutions${i}]
    set totalTime [subst \$_totalTime${i}]
    set totalSolTime [subst \$_totalSolTime${i}]
    set totalSoftTime [subst \$_totalSoftTime${i}]
    set totalScore [subst \$_totalScore${i}]
    set noErrors [subst \$_noErrors${i}]
    set totalStructRecall [subst \$_totalStructRecall${i}]
    set totalLexicalRecall [subst \$_totalLexicalRecall${i}]
    set totalLabelledRecall [subst \$_totalLabelledRecall${i}]
    
    $itk_component(totalTime${i}) configure -state normal
    $itk_component(totalSolTime${i}) configure -state normal
    $itk_component(totalSoftTime${i}) configure -state normal
    $itk_component(score${i}) configure -state normal
    $itk_component(noErrors${i}) configure -state normal
    $itk_component(totalStructRecall${i}) configure -state normal
    $itk_component(totalLexicalRecall${i}) configure -state normal
    $itk_component(totalLabelledRecall${i}) configure -state normal

    $itk_component(totalTime${i}) delete 0 end
    $itk_component(totalSolTime${i}) delete 0 end
    $itk_component(totalSoftTime${i}) delete 0 end
    $itk_component(score${i}) delete 0 end
    $itk_component(noErrors${i}) delete 0 end
    $itk_component(totalStructRecall${i}) delete 0 end
    $itk_component(totalLexicalRecall${i}) delete 0 end
    $itk_component(totalLabelledRecall${i}) delete 0 end

    $itk_component(score${i}) insert 0 [format "%.4f" $totalScore]
    $itk_component(noErrors${i}) insert 0 $noErrors
    $itk_component(totalTime${i}) insert 0 "[timeToString $totalTime]"
    $itk_component(totalSolTime${i}) insert 0 "[timeToString $totalSolTime]"
    $itk_component(totalSoftTime${i}) insert 0 "[timeToString $totalSoftTime]"
    $itk_component(totalStructRecall${i}) insert 0 [format "%.4f%%" $totalStructRecall]
    $itk_component(totalLexicalRecall${i}) insert 0 [format "%.4f%%" $totalLexicalRecall]
    $itk_component(totalLabelledRecall${i}) insert 0 [format "%.4f%%" $totalLabelledRecall]

    $itk_component(totalTime${i}) configure -state disabled
    $itk_component(totalSolTime${i}) configure -state disabled
    $itk_component(totalSoftTime${i}) configure -state disabled
    $itk_component(score${i}) configure -state disabled
    $itk_component(noErrors${i}) configure -state disabled
    $itk_component(totalStructRecall${i}) configure -state disabled
    $itk_component(totalLexicalRecall${i}) configure -state disabled
    $itk_component(totalLabelledRecall${i}) configure -state disabled
  }

  # redraw the judge canvas
  $itk_component(judgeBar) draw \
      $_fasterWordgraphs $_betterWordgraphs \
      $_betterAndFasterWordgraphs $_betterButSlowerWordgraphs $_softerWordgraphs \
      $_harderWordgraphs $_sameWordgraphs $_worseButFasterWordgraphs \
      $_worseAndSlowerWordgraphs $_worseWordgraphs $_slowerWordgraphs \
      $_weirdWordgraphs

}

## ----------------------------------------------------------------------------
## reload
## ----------------------------------------------------------------------------
body YadaDifference::reload {} {
  set selection [$itk_component(table) curselection]
  clear
  if {$selection != ""} {
    $itk_component(table) selection set [lindex $selection 0] [lindex $selection end]
  }
  loadWordgraphs
  _computeStatistics
}

## ----------------------------------------------------------------------------
## _getSelectedWordgraphs
## ----------------------------------------------------------------------------
body YadaDifference::_getSelectedWordgraphs {} {
  set table $itk_component(table)
  set cells [$table curselection]
  set selectedWordgraphs ""
  foreach index $cells {
    scan $index "%d,%d" row col
    if {$row == 0 || [info exists seen($row)]} {
      continue
    }
    set seen($row) 1
    set wordgraph [$table getCell $row,0]
    if {$wordgraph == ""} {
      continue
    }
    lappend selectedWordgraphs $wordgraph
  }

  return $selectedWordgraphs
}

## ----------------------------------------------------------------------------
## _insertData
## ----------------------------------------------------------------------------
body YadaDifference::_insertData {wordgraph args} {
  set table $itk_component(table)
  set cmax [$table cget -cols]

  # get place where we belong to
  set row [_getRowOfWordgraph $wordgraph]

  # fill it
  $table clear tags $row,0 $row,$cmax
  $table erase $row,1 $row,$cmax
  $table setCell row $row,1 $args

  # show it
  $table see $row,1
}

## ----------------------------------------------------------------------------
## load wordgraphs from their raw xml data files. if no wordgraphs are 
## given the currently active ones are taken
## ----------------------------------------------------------------------------
body YadaDifference::loadWordgraphs {{wordgraphs ""}} {

  # reset scores of selected wordgraphs
  if {$wordgraphs == ""} {
    set selectedWordgraphs [_getSelectedWordgraphs]
  } else {
    set selectedWordgraphs $wordgraphs
  }

  if {$selectedWordgraphs == ""} {
    set selectedWordgraphs [_getWordgraphs]
  }

  if {$selectedWordgraphs == ""} {
    return
  }
  
  _resetDataOf "$selectedWordgraphs"

  # load all xml files 
  set noLoadedFiles 0

  set experiments [_getSelectedExperiments]
  if {[llength $experiments] != 2} {
    return
  }
  set experiment1 [lindex $experiments 0]
  set experiment2 [lindex $experiments 1]

  # get data 
  set oldCursor [setBusy]
  foreach wordgraph $selectedWordgraphs {
    set file1 [$experiment1 getXmlFileName $wordgraph]
    set file2 [$experiment2 getXmlFileName $wordgraph]

    # check if the file is present
    if {![file exists $file1] || ![file exists $file2]} {
      continue;
    }

    # read
    foreach i {1 2} {
      set file [subst \$file$i]
      set fd [open "|zcat $file" r]
      set data [read $fd]
      close $fd
      if {[catch {
	set document [dom parse $data]
	_analyseDocument $document [subst \$experiment$i]
	$document delete
	incr noLoadedFiles
	set document ""
      } errMsg]} {
	_print "ERROR: invalid xml file $file"
      } 
    }
    update
  }
  resetBusy $oldCursor
  
  if {$noLoadedFiles > 0} {
    setModified
  }
  _resortRows
  #_computeStatistics
}

## ----------------------------------------------------------------------------
## _getRowOfWordgraph
## ----------------------------------------------------------------------------
body YadaDifference::_getRowOfWordgraph {wordgraph} {

  set table $itk_component(table)
  
  set rmax [$table cget -rows]
  set ids [$table getCell 1,0 $rmax,0]
  set row [lsearch -exact $ids $wordgraph]
  incr row

  return $row
}

## ----------------------------------------------------------------------------
## _analyseDocument
## ----------------------------------------------------------------------------
body YadaDifference::_analyseDocument {document experiment} {

  # default analysis
  array set _analyseResult {
    wordgraph		""
    noValues		0
    score		0.0
    totalTime		0
    solutionTime	0
    softTime		0
    tries		0
    structHits		0
    labelledHits	0
    lexicalHits		0
    strictHits		0
  }

  # get the root element
  set cdgpNode [$document documentElement]
  if {$cdgpNode == ""} {
    _print "ERROR: no cdgp logfile"
    return 0
  } 

  # xpath analysis
  
  # get number of values
  if {[catch {
    set _analyseResult(noValues) [[$cdgpNode selectNodes \
      {descendant::netInfo[1]/number[@name="noValues"]}] @value ]}]} {
    print "ERROR: no net information found => newnet failed"
    return 0
  }
    
  # get name of wordgraph
  if {[catch {
    set _analyseResult(wordgraph) [[$cdgpNode selectNodes \
      {descendant::netInfo[1]/ref[@type="Lattice"]}] @name ]}]} {
    print "ERROR: no net information found => newnet failed"
    return 0
  }

  # get the score
  if {[catch {set _analyseResult(score) [[$cdgpNode selectNodes \
    {descendant::parse[1]/number[@name="score"]}] @value]}]} {
    print "WARNING: no parse found\n"
  }

  # get the parse verification node
  set parseVerificationNode [$cdgpNode descendant 1 "parseverification"]
  if {$parseVerificationNode != ""} {
    foreach aspect {tries structHits labelledHits lexicalHits strictHits} {
      set _analyseResult($aspect) [[$parseVerificationNode selectNodes \
	descendant::[string tolower $aspect]/number\[@name="SYN"\]] @value]
    }
  } else {
    # verification fallback: compute tries from annotation
    _print "WARING: no parse verification found, consulting annotation"
    set grammar [.main getGrammar [$experiment cget -grammarName]]
    $grammar scan; # CAUTION: might be costy
    set yadaWordgraph [$grammar getWordgraph $_analyseResult(wordgraph)]
    set anno [$yadaWordgraph getBestAnnotation]
    if {$anno == ""} {
      _print "WARNING: no annotation for wordgraph '$_analyseResult(wordgraph)'"
    } else {
      set _analyseResult(tries) [$anno cget -noEdges]
    }
  }

  # parse the statistics 
  foreach aspect {totalTime solutionTime softTime} {
    set node [$cdgpNode selectNodes \
      descendant::statistics/number\[@name="$aspect"\]]
    if {$node == ""} {
      continue;
    }
    if {[llength $node] > 1} {
      set node [lindex $node 1]
    }
    set _analyseResult($aspect) [$node getAttribute "value"]
    if {$_analyseResult($aspect) == ""} {
      set _analyseResult($aspect) 0
    }
  }

  # end of xpath analysis

  # fix solution and soft time in case of no score
  if {$_analyseResult(softTime) == 0} {
    set _analyseResult(softTime) $_analyseResult(solutionTime)
  }
  if {$_analyseResult(solutionTime) == 0} {
    set _analyseResult(softTime) $_analyseResult(totalTime)
    set _analyseResult(solutionTime) $_analyseResult(totalTime)
  }
  

  # the wordgraph is needed at least
  set wordgraph $_analyseResult(wordgraph)
  if {$wordgraph == ""} {
    _print "ERROR: no information available in this run"
    return 0
  }

  # compute derived data
  set row [_getRowOfWordgraph $wordgraph]
  set dataRow [$itk_component(table) getCell $row,3 $row,13]

  set noErrors1		[lindex $dataRow 0]
  set time1		[lindex $dataRow 1]
  set solutionTime1	[lindex $dataRow 2]
  set softTime1		[lindex $dataRow 3]
  set noErrors2		[lindex $dataRow 6]
  set time2		[lindex $dataRow 7]
  set solutionTime2	[lindex $dataRow 8]
  set softTime2		[lindex $dataRow 9]

  if {$noErrors1 == ""} {
    set noErrors1 0
  }
  if {$time1 == ""} {
    set time1 0
  }
  if {$solutionTime1 == ""} {
    set solutionTime1 0
  }
  if {$softTime1 == ""} {
    set softTime1 0
  }
  if {$noErrors2 == ""} {
    set noErrors2 0
  }
  if {$time2 == ""} {
    set time2 0
  }
  if {$solutionTime2 == ""} {
    set solutionTime2 0
  }
  if {$softTime2 == ""} {
    set softTime2 0
  }

  if {$_triesOf($wordgraph,1) == 0} {
    set lexicalRecall1 0
  } else {
    set lexicalRecall1 [expr $_lexicalHitsOf($wordgraph,1) * 100.0 / $_triesOf($wordgraph,1)]
  }
  if {$_triesOf($wordgraph,2) == 0} {
    set lexicalRecall2 0
  } else {
    set lexicalRecall2 [expr $_lexicalHitsOf($wordgraph,2) * 100.0 / $_triesOf($wordgraph,2)]
  }

  # save score and edge data
  if {[$itk_component(experiment1Box) get] == [$experiment cget -name]} {
    set experimentNo 1
  } else {
    set experimentNo 2
  }
  set _scoreOf($wordgraph,$experimentNo) $_analyseResult(score)
  set _structHitsOf($wordgraph,$experimentNo) $_analyseResult(structHits)
  set _lexicalHitsOf($wordgraph,$experimentNo) $_analyseResult(lexicalHits)
  set _labelledHitsOf($wordgraph,$experimentNo) $_analyseResult(labelledHits)
  set _triesOf($wordgraph,$experimentNo) $_analyseResult(tries)

  # overwrite the given experiment data
  set score1 $_scoreOf($wordgraph,1)
  set score2 $_scoreOf($wordgraph,2)
  set time${experimentNo} $_analyseResult(totalTime)
  set solutionTime${experimentNo} $_analyseResult(solutionTime)
  set softTime${experimentNo} $_analyseResult(softTime)
  set noErrors${experimentNo} [expr $_analyseResult(tries) - $_analyseResult(lexicalHits) ]
  if {$_analyseResult(tries) > 0} {
    set lexicalRecall${experimentNo} [expr $_analyseResult(lexicalHits) * 100.0 / $_analyseResult(tries)]
  }

  set speedUp [_computeSpeedup $time1 $time2]
  set solSpeedUp [_computeSpeedup $solutionTime1 $solutionTime2]
  set softSpeedUp [_computeSpeedup $softTime1 $softTime2]
  set judge [_judgeData \
    $lexicalRecall1 $score1 $time1 $solutionTime1 $softTime1 \
    $lexicalRecall2 $score2 $time2 $solutionTime2 $softTime2]
  if {$judge == "weird"} {
    #_print "ERROR: wordgraph '$wordgraph' is weird: score1=$score1 score2=$score2 lexicalRecall1=$lexicalRecall1 lexicalRecall2=$lexicalRecall2"
  }

  # add the collected data to the table
  _insertData $wordgraph \
	      $_analyseResult(noValues) \
	      [format "%4.3e" $score1] \
	      $noErrors1 \
	      $time1 \
	      $solutionTime1 \
	      $softTime1 \
	      [format "%3.2f" $lexicalRecall1] \
	      [format "%4.3e" $score2] \
	      $noErrors2 \
	      $time2 \
	      $solutionTime2 \
	      $softTime2 \
	      [format "%3.2f" $lexicalRecall2] \
	      [format "%.2g" $speedUp] \
	      [format "%.2g" $solSpeedUp] \
	      [format "%.2g" $softSpeedUp] \
	      $judge
}

## ----------------------------------------------------------------------------
## _resetDataOf
## ----------------------------------------------------------------------------
body YadaDifference::_resetDataOf {wordgraphs} {

  foreach wordgraph $wordgraphs {
    foreach i {1 2} {
      set _scoreOf($wordgraph,$i) 0.0
      set _structHitsOf($wordgraph,$i) 0
      set _lexicalHitsOf($wordgraph,$i) 0
      set _labelledHitsOf($wordgraph,$i) 0
      set _triesOf($wordgraph,$i) 0
    }
  }
}

## ----------------------------------------------------------------------------
## _computeSpeedup
## ----------------------------------------------------------------------------
body YadaDifference::_computeSpeedup {time1 time2} {

  set timeTolerance [.main getPreference timeTolerance]

  if {[expr abs ($time1 - $time2) > $timeTolerance]} {
    return [expr (1.0 + $time2) / (1.0 + $time1)]
  } else {
    return 1.0
  }
}

## ----------------------------------------------------------------------------
## _judgeData
## ----------------------------------------------------------------------------
body YadaDifference::_judgeData {measure1a measure1b time1 solutionTime1 softTime1 measure2a measure2b time2 solutionTime2 softTime2} {

  set significance [.main getPreference significance]

  # is there an annotation? if not then
  if {$measure1a == 0 && $measure2a == 0} {
    # ... use scores
    set measure1 $measure1b
    set measure2 $measure2b
  } else {
    # ... otherwise use recall
    set measure1 $measure1a
    set measure2 $measure2a
  }

  # compare properties
  set measureDiff [expr $measure1 - $measure2 ]
  set speedUp [_computeSpeedup $time1 $time2]
  set solSpeedUp [_computeSpeedup $solutionTime1 $solutionTime2]
  set softSpeedUp [_computeSpeedup $softTime1 $softTime2]

  # judge
  if {$measureDiff > 1e-5} {
    if {$solutionTime2 != 0.0} { ;# does experiment 2 give us anything
      if {$speedUp < [expr 1.0 / $significance]} {
	set judge "betterButSlower"
      } elseif {$speedUp > $significance} {
	set judge "betterAndFaster"
      } else {
	set judge "better"
      }
    } else {
      set judge "better"
    }
  } elseif {$measureDiff < -1e-5} {
    if {$solutionTime1 != 0.0} { ;# does experiment 1 give us anything
      if {$speedUp > $significance} {
	set judge "worseButFaster"
      } elseif {$speedUp < [expr 1.0 / $significance]} {
	set judge "worseAndSlower"
      } else {
	set judge "worse"
      }
    } else {
      set judge "worse"
    }
  } elseif {$speedUp < [expr 1.0 / $significance]} {
    set judge "slower"
  } elseif {$speedUp > $significance} {
    set judge "faster"
  } elseif {$solSpeedUp > $significance} {
    set judge "softer"
  } elseif {$solSpeedUp < [expr 1.0 / $significance]} {
    set judge "harder"
  } elseif {$softSpeedUp > $significance} {
    set judge "softer"
  } elseif {$softSpeedUp < [expr 1.0 / $significance]} {
    set judge "harder"
  } else {
    set judge "same"
  }

  # weirdness check, this relys on measure1b and measure2b to be scores
  # measure a: recall
  # measure b: score
  if {[.main getPreference weirdness]} {
    if {(
      	  $measure1a == 100.0 &&			
  	  $measure2a < [expr $measure1a - 1e-10] &&
  	  $measure1b <= $measure2b                
    	) || (
  	  $measure2a == 100.0 &&			
  	  $measure1a < [expr $measure2a - 1e-10] &&
  	  $measure2b <= $measure1b                
    	)} {
#      set judge "weird"
    } 
  } else {
    if {(
	  $measure2b != 0.0 &&
	  $measure2a < [expr $measure1a - 1e-10] &&
	  $measure1b <= $measure2b                
	) || (
	  $measure1b != 0.0 &&
	  $measure1a < [expr $measure2a - 1e-10] &&
	  $measure2b <= $measure1b                
	)} {
#      set judge "weird"
    }
  }

  return $judge
}

## ----------------------------------------------------------------------------
## _getSelectedExperiments
## ----------------------------------------------------------------------------
body YadaDifference::_getSelectedExperiments {} {

  set experiments ""
  set experimentName [$itk_component(experiment1Box) get]
  if {$experimentName != "" && $experimentName != "<none>"} {
    lappend experiments [.main getExperiment $experimentName]
  }

  set experimentName [$itk_component(experiment2Box) get]
  if {$experimentName != "" && $experimentName != "<none>"} {
    lappend experiments [.main getExperiment $experimentName]
  }

  return $experiments
}

## ----------------------------------------------------------------------------
## _getWordgraphs
## ----------------------------------------------------------------------------
body YadaDifference::_getWordgraphs {} {

  set experiments [_getSelectedExperiments]
  if {[llength $experiments] != 2} {
    return ""
  }
  
  set experiment1 [lindex $experiments 0]
  set experiment2 [lindex $experiments 1]
  set grammarName1 [$experiment1 cget -grammarName]
  set grammarName2 [$experiment2 cget -grammarName]
  set grammar1 [.main getGrammar $grammarName1]
  set selection [$grammar1 getSelection]

  # shortcut for two experiments using the same grammar
  if {$grammarName1 == $grammarName2} {
    return $selection
  }

  set grammar2 [.main getGrammar $grammarName2]
  
  # compute intersection of the used wordgraphs
  set commonWordgraphs ""
  foreach wgName $selection {
    if {[$grammar isSelected $wgName]} {
      lappend commonWordgraphs $wgName
    }
  }

  return $commonWordgraphs
}

## ----------------------------------------------------------------------------
## _resortRows
## ----------------------------------------------------------------------------
body YadaDifference::_resortRows {} {
  if {$_lastSorting != ""} {
    eval _sortRows $_lastSorting
  }
}

## ----------------------------------------------------------------------------
## _sortRows
## ----------------------------------------------------------------------------
body YadaDifference::_sortRows {colNo args} {
  set type [_getTypeOfColumn $colNo]
  eval $itk_component(table) sortRows $colNo -$type $args
  set _lastSorting $colNo
  eval lappend _lastSorting $args
}

## ----------------------------------------------------------------------------
## _print
## ----------------------------------------------------------------------------
body YadaDifference::_print {message} {
  .main printStatus "$message"
}

## ----------------------------------------------------------------------------
## _getTypeOfColumn
## ----------------------------------------------------------------------------
body YadaDifference::_getTypeOfColumn {colNo} {
  return $_typeOfColumn($colNo)
}

## ----------------------------------------------------------------------------
## _showContextMenu
## ----------------------------------------------------------------------------
body YadaDifference::_showContextMenu {x y xroot yroot} {
  set _selectedContextWordgraphs [_getSelectedWordgraphs]
  if {$_selectedContextWordgraphs == ""} {
    set _selectedContextWordgraphs [$itk_component(table) getCell @0,$y]
  }
  if {$_selectedContextWordgraphs == ""} {
    return
  }

  tk_popup $itk_component(contextMenu) $xroot $yroot
}

## ----------------------------------------------------------------------------
## _clearRow
## ----------------------------------------------------------------------------
body YadaDifference::_clearRow {} {

  set table $itk_component(table)
  set cmax [$table cget -cols]

  foreach wordgraph $_selectedContextWordgraphs {
    set row [_getRowOfWordgraph $_selectedContextWordgraphs]
    $table clear tags $row,0 $row,$cmax
    $table erase $row,1 $row,$cmax
  }

  set _selectedContextWordgraphs ""
}

## ----------------------------------------------------------------------------
## _viewLogfile
## ----------------------------------------------------------------------------
body YadaDifference::_viewLogfile {} {

  set experiments [_getSelectedExperiments]
  if {[llength $experiments] == 0} {
    return
  }

  set experiment1 [lindex $experiments 0]
  foreach wordgraph $_selectedContextWordgraphs {
    # first
    set title "Yada.Log - $wordgraph"
    set fileName [$experiment1 getLogFileName $wordgraph]
    set yadaLog [viewFile $fileName $title]

    # second
    if {[llength $experiments] > 1} {
      set experiment2 [lindex $experiments 1]
      set fileName [$experiment2 getLogFileName $wordgraph]
      if {[file exists $fileName]} {
	if {$yadaLog == ""} {
	  set yadaLog [viewFile $fileName $title]
	} else {
	  $yadaLog print "\n\nINFO: loading $fileName\n"
	  $yadaLog print [$experiment2 getLogFile $wordgraph]
	  $yadaLog see 0.0
	}
      }
    }
    if {$yadaLog != ""} {
      $yadaLog activate
    }
  }
  
  set _selectedContextWordgraphs ""
}

## ----------------------------------------------------------------------------
## _viewXmlfile
## ----------------------------------------------------------------------------
body YadaDifference::_viewXmlfile {} {

  set experiments [_getSelectedExperiments]
  if {[llength $experiments] == 0} {
    return
  }

  foreach wordgraph $_selectedContextWordgraphs {
    # first
    set experiment1 [lindex $experiments 0]
    set title "Yada.Xml - $wordgraph ([$experiment1 cget -name])"
    set fileName [$experiment1 getXmlFileName $wordgraph]
    set yadaLog [viewFile $fileName $title]

    # second
    if {[llength $experiments] > 1} {
      set experiment2 [lindex $experiments 1]
      set fileName [$experiment2 getXmlFileName $wordgraph]
      if {[file exists $fileName]} {
	if {$yadaLog == ""} {
	  set yadaLog [viewFile $fileName $title]
	} else {
	  $yadaLog configure -title \
	    "Yada.Xml - $wordgraph ([$experiment1 cget -name] vs. [$experiment2 cget -name])"
	  $yadaLog print "\n\nINFO: loading $fileName\n"
	  $yadaLog print [$experiment2 getXmlFile $wordgraph]
	  $yadaLog see 0.0
	}
      }
    }
    if {$yadaLog != ""} {
      $yadaLog activate
    }
  }
  
  set _selectedContextWordgraphs ""
}

## ----------------------------------------------------------------------------
## get the title string representing the state of this document
## ----------------------------------------------------------------------------
body YadaDifference::getWindowTitle {} {

  set experiments [_getSelectedExperiments]
  set experiment1 [lindex $experiments 0]
  set experiment2 [lindex $experiments 1]
  set experiment1Name ""
  set experiment2Name ""

  if {$experiment1 != "" && $experiment1 != "<none>"} {
    set experiment1Name [$experiment1 cget -name]
  }
  if {$experiment2 != "" && $experiment2 != "<none>"} {
    set experiment2Name [$experiment2 cget -name]
  }

  if {$experiment1Name != "" && $experiment2Name != ""} {
    return "[chain]: $experiment1Name vs. $experiment2Name"
  } 

  return "[chain]"
}

## ----------------------------------------------------------------------------
## @returns true if the selected item is not named <none> but has been changed
## ----------------------------------------------------------------------------
body YadaDifference::isModified {} {
  set experiments [_getSelectedExperiments]
  set experiment1 [lindex $experiments 0]
  set experiment2 [lindex $experiments 1]

  if {$experiment1 == "" || [$experiment1 cget -name] == "<none>" ||
      $experiment2 == "" || [$experiment2 cget -name] == "<none>"} {
    return 0
  }
  return [chain]
}

## ----------------------------------------------------------------------------
## serializes this runmner into a dom document
## @returns a new dom document
## ----------------------------------------------------------------------------
body YadaDifference::toDom {} {

  set document [dom createDocument "yada"]
  set root [$document documentElement]

  set differenceNode [$document createElement "difference"]
  $root appendChild $differenceNode

  set experiment1Name [$itk_component(experiment1Box) get]
  set experiment2Name [$itk_component(experiment2Box) get]

  $differenceNode setAttribute "name" "$experiment1Name vs. $experiment2Name"

  set experiment1Node [$document createElement "experimentRef"]
  $differenceNode appendChild $experiment1Node
  $experiment1Node setAttribute "no" "1"
  $experiment1Node setAttribute "name" "$experiment1Name"

  set experiment2Node [$document createElement "experimentRef"]
  $differenceNode appendChild $experiment2Node
  $experiment2Node setAttribute "no" "2" 
  $experiment2Node setAttribute "name" "$experiment2Name"
  
  # experiment 1
  set statsNode [$document createElement "stats"]
  $experiment1Node appendChild $statsNode
  $statsNode setAttribute "score" $_totalScore1 
  $statsNode setAttribute "numErrors" $_noErrors1 
  $statsNode setAttribute "numSolutions" $_noSolutions1 
  $statsNode setAttribute "time" $_totalTime1
  $statsNode setAttribute "solTime" $_totalSolTime1 
  $statsNode setAttribute "softTime" $_totalSoftTime1 
  $statsNode setAttribute "structRecall" $_totalStructRecall1 
  $statsNode setAttribute "labelRecall" $_totalLabelledRecall1 
  $statsNode setAttribute "lexicalRecall" $_totalLexicalRecall1

  # experiment 2
  set statsNode [$document createElement "stats"]
  $experiment2Node appendChild $statsNode
  $statsNode setAttribute "score" $_totalScore2 
  $statsNode setAttribute "numErrors" $_noErrors2 
  $statsNode setAttribute "numSolutions" $_noSolutions2 
  $statsNode setAttribute "time" $_totalTime2 
  $statsNode setAttribute "solTime" $_totalSolTime2
  $statsNode setAttribute "softTime" $_totalSoftTime2
  $statsNode setAttribute "structRecall" $_totalStructRecall2
  $statsNode setAttribute "labelRecall" $_totalLabelledRecall2
  $statsNode setAttribute "lexicalRecall" $_totalLexicalRecall2

  # judge
  set judgeNode [$document createElement "judge"]
  $differenceNode appendChild $judgeNode
  $judgeNode setAttribute "faster" $_fasterWordgraphs 
  $judgeNode setAttribute "better" $_betterWordgraphs
  $judgeNode setAttribute "betterAndFaster" $_betterAndFasterWordgraphs
  $judgeNode setAttribute "betterButSlower" $_betterButSlowerWordgraphs
  $judgeNode setAttribute "softer" $_softerWordgraphs
  $judgeNode setAttribute "harder" $_harderWordgraphs
  $judgeNode setAttribute "same" $_sameWordgraphs
  $judgeNode setAttribute "worseButFaster" $_worseButFasterWordgraphs
  $judgeNode setAttribute "worseAndSlower" $_worseAndSlowerWordgraphs
  $judgeNode setAttribute "worse" $_worseWordgraphs
  $judgeNode setAttribute "slower" $_slowerWordgraphs
  $judgeNode setAttribute "weird" $_weirdWordgraphs

  # wordgraphs
  set table $itk_component(table)
  set rmax [$table cget -rows]
  set cmax [$table cget -cols] 
  for {set row 1} {$row < $rmax} {incr row} {
    set dataRow [$table getCell $row,0 $row,$cmax]
    if {[lindex $dataRow 2] == ""} {
      continue
    }

    set wordgraphNode [$document createElement "wordgraph"]
    set wordgraphName [lindex $dataRow 0]
    $differenceNode appendChild $wordgraphNode
    $wordgraphNode setAttribute "name" $wordgraphName 
    $wordgraphNode setAttribute "values" [lindex $dataRow 1]
    $wordgraphNode setAttribute "score1" [lindex $dataRow 2]
    $wordgraphNode setAttribute "errors1" [lindex $dataRow 3]
    $wordgraphNode setAttribute "time1" [lindex $dataRow 4]
    $wordgraphNode setAttribute "solTime1" [lindex $dataRow 5]
    $wordgraphNode setAttribute "softTime1" [lindex $dataRow 6]
    $wordgraphNode setAttribute "lexicalRecall1" [lindex $dataRow 7]
    $wordgraphNode setAttribute "structHits1" $_structHitsOf($wordgraphName,1)
    $wordgraphNode setAttribute "labelledHits1" $_labelledHitsOf($wordgraphName,1)
    $wordgraphNode setAttribute "lexicalHits1" $_lexicalHitsOf($wordgraphName,1)
    $wordgraphNode setAttribute "tries1" $_triesOf($wordgraphName,1)
    $wordgraphNode setAttribute "score2" [lindex $dataRow 8]
    $wordgraphNode setAttribute "errors2" [lindex $dataRow 9]
    $wordgraphNode setAttribute "time2" [lindex $dataRow 10]
    $wordgraphNode setAttribute "solTime2" [lindex $dataRow 11]
    $wordgraphNode setAttribute "softTime2" [lindex $dataRow 12]
    $wordgraphNode setAttribute "lexicalRecall2" [lindex $dataRow 13]
    $wordgraphNode setAttribute "structHits2" $_structHitsOf($wordgraphName,2)
    $wordgraphNode setAttribute "labelledHits2" $_labelledHitsOf($wordgraphName,2)
    $wordgraphNode setAttribute "lexicalHits2" $_lexicalHitsOf($wordgraphName,2)
    $wordgraphNode setAttribute "tries2" $_triesOf($wordgraphName,2)
    $wordgraphNode setAttribute "speedUp" [lindex $dataRow 14]
    $wordgraphNode setAttribute "solSpeedUp" [lindex $dataRow 15]
    $wordgraphNode setAttribute "softSpeedUp" [lindex $dataRow 16]
    $wordgraphNode setAttribute "judge" [lindex $dataRow 17]
  }

  return $document
}

## ----------------------------------------------------------------------------
## get the filename where the persistent state is saved in
## ----------------------------------------------------------------------------
body YadaDifference::getFileName {} { 
  global env

  set experiment1Name [$itk_component(experiment1Box) get]
  set experiment2Name [$itk_component(experiment2Box) get]

  if {$experiment1Name == "" || $experiment1Name == "<none>" ||
      $experiment2Name == "" || $experiment2Name == "<none>"} {
    return ""
  }
  
  set baseName "${experiment1Name}_${experiment2Name}_${name}"
  set fileName [file join $env(YADA_DATA) $baseName.xml]
  return [getSecureFileName $fileName]
}

## ----------------------------------------------------------------------------
## initialize this document from a dom node
## @param document containing a /yada/difference
## ----------------------------------------------------------------------------
body YadaDifference::fromDom {document} {
  set rootNode [$document documentElement]
  set diffNode [$rootNode selectNodes "/yada/difference"]
  fromDomNode $diffNode
}

## ----------------------------------------------------------------------------
## initialize this runner from a dom node
## @param diffNode containing a difference description
## \TODO this refreshes the display also -> should be a separate step
## ----------------------------------------------------------------------------
body YadaDifference::fromDomNode {diffNode} {

  set experiments [_getSelectedExperiments]
  if {$experiments == ""} {
    return
  }
  set experiment1 [lindex $experiments 0]
  set experiment2 [lindex $experiments 1]
  if {$experiments == ""} {
    return
  }
  if {$experiment1 == "" || $experiment2 == ""} {
    return
  }

  # get the timestamp for this document
  file stat [getZippedFileName] statDocumentFile

  # get set of wordgraphs to be loaded
  set activeWordgraphs [_getSelectedWordgraphs]
  if {$activeWordgraphs == ""} {
    clear
    set activeWordgraphs [_getWordgraphs]
  }

  # read wordgraphs from the document's xml file
  set updatedWordgraphs {}
  foreach wordgraph $activeWordgraphs {

    # get description
    set node [$diffNode selectNodes "./wordgraph\[@name=\"$wordgraph\"\]" 0]

    # if there's none then load this wordgraph from its own file
    if {$node == ""} {
      # remember this node to be loaded
      lappend updatedWordgraphs $wordgraph
      continue
    }

    # only load wordgraphs from the document file, 
    # if there is no updated wordgraph xml file
    set wordgraphFileName [$experiment1 getXmlFileName $wordgraph]
    if {![catch {file stat $wordgraphFileName statWordgraphFile}]} {
      if {[expr {$statDocumentFile(ctime) < $statWordgraphFile(ctime)}]} {
	lappend updatedWordgraphs $wordgraph
	continue
      }
    }
    set wordgraphFileName [$experiment2 getXmlFileName $wordgraph]
    if {![catch {file stat $wordgraphFileName statWordgraphFile}]} {
      if {[expr {$statDocumentFile(ctime) < $statWordgraphFile(ctime)}]} {
	lappend updatedWordgraphs $wordgraph
	continue
      }
    }

    #  <wordgraph name="n001k000-1" values="452" 
    #  score1="6.570e-01" errors1="0" time1="694" solTime1="63" softTime1="13" lexicalRecall1="100.00" 
    #  structHits1="9" labelledHits1="9" lexicalHits1="9" tries1="9" 
    #  score2="6.570e-01" errors2="0" time2="738" solTime2="98" softTime2="14" lexicalRecall2="100.00" 
    #  structHits2="9" labelledHits2="9" lexicalHits2="9" tries2="9"
    # speedUp="1" solSpeedUp="1" softSpeedUp="1" judge="same"/>

    set values [$node getAttribute "values"]

    set score1 [$node getAttribute "score1"]
    set errors1 [$node getAttribute "errors1"]
    set time1 [$node getAttribute "time1"]
    set solTime1 [$node getAttribute "solTime1"]
    set softTime1 [$node getAttribute "softTime1"]
    set lexicalRecall1 [$node getAttribute "lexicalRecall1"]
    set _structHitsOf($wordgraph,1) [$node getAttribute "structHits1"]
    set _labelledHitsOf($wordgraph,1) [$node getAttribute "labelledHits1"]
    set _lexicalHitsOf($wordgraph,1) [$node getAttribute "lexicalHits1"]
    set _triesOf($wordgraph,1) [$node getAttribute "tries1"]
    set _scoreOf($wordgraph,1) $score1

    set score2 [$node getAttribute "score2"]
    set errors2 [$node getAttribute "errors2"]
    set time2 [$node getAttribute "time2"]
    set solTime2 [$node getAttribute "solTime2"]
    set softTime2 [$node getAttribute "softTime2"]
    set lexicalRecall2 [$node getAttribute "lexicalRecall2"]
    set _structHitsOf($wordgraph,2) [$node getAttribute "structHits2"]
    set _labelledHitsOf($wordgraph,2) [$node getAttribute "labelledHits2"]
    set _lexicalHitsOf($wordgraph,2) [$node getAttribute "lexicalHits2"]
    set _triesOf($wordgraph,2) [$node getAttribute "tries2"]
    set _scoreOf($wordgraph,2) $score2

    set speedUp [$node getAttribute "speedUp"]
    set solSpeedUp [$node getAttribute "solSpeedUp"]
    set softSpeedUp [$node getAttribute "softSpeedUp"]
    set judge [$node getAttribute "judge"]

    _insertData $wordgraph \
		$values \
		[format "%4.3e" $score1] \
		$errors1 \
		$time1 \
		$solTime1 \
		$softTime1 \
		[format "%3.2f" $lexicalRecall1] \
		[format "%4.3e" $score2] \
		$errors2 \
		$time2 \
		$solTime2 \
		$softTime2 \
		[format "%3.2f" $lexicalRecall2] \
		[format "%.2g" $speedUp] \
		[format "%.2g" $solSpeedUp] \
		[format "%.2g" $softSpeedUp] \
		$judge
    
    update
  }
  if {$updatedWordgraphs != ""} {
    loadWordgraphs $updatedWordgraphs
  } else {
    resetModified
  }
  _computeStatistics
}
