# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaPreferences - Container for the application level perferences.
## \ingroup YadaConfigDocument
##
## \author Michael Daum
##
## $Id: YadaGenerals.tcl,v 1.18 2005-12-01 16:23:27 micha Exp $
## ----------------------------------------------------------------------------
class YadaPreferences {
  inherit itk::Widget YadaPlainDocument


  # variables ----------------------------------------------------------------

  ## percentage indicating a significant difference in a quotient of two numbers.
  public variable significance  1.1

  ## tolerated time difference which isn't significant for a profiling distinction
  public variable timeTolerance 500

  ## the name of an editor to launch when we want to edit something
  public variable editor ""

  ## flag indicating how to compute weirdness in the YadaRunnableDocument s.
  public variable weirdness 1

  # methods ------------------------------------------------------------------
  public method init {}
  public method toDom {}
  public method fromDom {document}; ## \type domDoc
  public method fromDomNode {globalNode}; ## \type domNode
  public method updateMenus {}
  public method getFileName {}; ## \virtual
  public method activationHandle {}

  constructor {args} {}; ## \type TclList

  private method _commit {}
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaPreferences::constructor {args} {

  # create custom file menu
  set fileButton [.main component fileButton]
  for {
    set i 1
    set fileMenu $fileButton.menu$i
  } {[winfo exists $fileMenu]} {incr i} {
    set fileMenu $fileButton.menu$i
  }
  itk_component add fileMenu {
      menu $fileMenu
  } {
      keep -background -cursor
      rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }

  ## create submenu entry "New"
  itk_component add newMenu {
     menu $itk_component(fileMenu).newMenu 
  } {
     keep -background -cursor
     rename -borderwidth -menuborderwidth menuBorderwidth Borderwidth
  }

  ## add file menu entries
  $itk_component(fileMenu) add command \
       	-label "Load" \
       	-underline 0 \
       	-command [code $this load] 
  $itk_component(fileMenu) add command \
       	-label "Save" \
       	-underline 0 \
       	-command [code $this save] 
  $itk_component(fileMenu) add cascade \
	-label "New" \
	-underline 0 \
	-menu $itk_component(newMenu)
  $itk_component(fileMenu) add separator
  $itk_component(fileMenu) add command \
	-label "Quit" \
	-underline 0 \
	-command {.main quit}


  $itk_component(newMenu) add command \
	-label "Runner" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaRunner Runner}
  $itk_component(newMenu) add command \
	-label "Difference" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaDifference Difference}
  $itk_component(newMenu) add command \
	-label "Gls Statistics" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaGlsStats GlsStatistics}
  $itk_component(newMenu) add command \
	-label "Ranking" \
	-underline 0 \
	-command {YadaDocument::newDocument YadaRanking Ranking}


  # the document's name
  set name "Preferences"

  itk_component add frame {
    frame $itk_interior.frame \
      -borderwidth 0
  }
  
  itk_component add significanceLabel {
    label $itk_component(frame).significanceLabel \
      -text "Significance:" \
      -anchor nw
  }

  itk_component add significance {
    entry $itk_component(frame).significance \
      -textvariable [scope significance] \
      -borderwidth 2 \
      -state normal \
      -width 20 \
  } {}

  itk_component add timeToleranceLabel {
    label $itk_component(frame).timeToleranceLabel \
      -text "Time Tolerance:" \
      -anchor nw
  }

  itk_component add timeTolerance {
    entry $itk_component(frame).timeTolerance \
      -textvariable [scope timeTolerance] \
      -borderwidth 2 \
      -state normal \
      -width 20 \
  } {}

  itk_component add editorLabel {
    label $itk_component(frame).editorLabel \
      -text "Editor:" \
      -anchor nw
  }

  itk_component add editor {
    entry $itk_component(frame).editor \
      -textvariable [scope editor] \
      -borderwidth 2 \
      -state normal \
      -width 20 \
  } {}

  itk_component add weirdnessLabel {
    label $itk_component(frame).weirdnessLabel \
      -text "Weirdness:" -anchor nw
  } {}

  itk_component add weirdness {
    checkbutton $itk_component(frame).strictWeirdness \
      -text "strict" \
      -variable [scope weirdness] \
      -command [code $this _commit]
  } {}


  # binding
  bind $itk_component(timeTolerance) <Return> [code $this _commit]
  bind $itk_component(significance) <Return> [code $this _commit]
  bind $itk_component(editor) <Return> [code $this _commit]

  # packing + gridding
  pack $itk_component(frame) -fill both -expand 1 -padx 10 -pady 10
  grid columnconfigure $itk_component(frame) 0 -pad 10
  grid columnconfigure $itk_component(frame) 1 -weight 1
  grid rowconfigure $itk_component(frame) 4 -weight 1

  grid $itk_component(editorLabel) -sticky nw -row 0 -column 0 -pady 10 
  grid $itk_component(editor) -sticky nw -row 0 -column 1 -pady 10 

  grid $itk_component(significanceLabel) -sticky ew -row 1 -column 0 -pady 10
  grid $itk_component(significance) -sticky nw -row 1 -column 1 -pady 10 

  grid $itk_component(timeToleranceLabel) -sticky ew -row 2 -column 0 -pady 10
  grid $itk_component(timeTolerance) -sticky nw -row 2 -column 1 -pady 10 

  grid $itk_component(weirdnessLabel) -sticky ew -row 3 -column 0 -pady 10
  grid $itk_component(weirdness) -sticky nw -row 3 -column 1 -pady 10 


  eval itk_initialize $args
  
}

## ----------------------------------------------------------------------------
## init
## ----------------------------------------------------------------------------
body YadaPreferences::init {} {
  if {$_isInitialized} {
      return
  }
  chain
  set isModified 1
  load
  if {$editor == ""} {
    global env
    configure -editor $env(YADA_EDITOR)
  }
}

## ----------------------------------------------------------------------------
## _commit
## ----------------------------------------------------------------------------
body YadaPreferences::_commit {} {
  configure \
    -significance $significance \
    -timeTolerance $timeTolerance \
    -editor $editor \
    -weirdness $weirdness
    -isModified 1
}

## ----------------------------------------------------------------------------
## Serializes Object into a dom document
## @returns a new dom document
## \TODO store all known objects and their xml file
## ----------------------------------------------------------------------------
body YadaPreferences::toDom {} {
  set document [dom createDocument "yada"]
  set rootNode [$document documentElement]
  set globalNode [$document createElement "global"]
  $rootNode appendChild $globalNode

  # save global preferences
  $globalNode setAttribute "significance" $significance
  $globalNode setAttribute "timeTolerance" $timeTolerance
  $globalNode setAttribute "editor" $editor
  $globalNode setAttribute "weirdness" $weirdness

  # save all grammars refs
  set grammarsNode [$document createElement "grammars"]
  $globalNode appendChild $grammarsNode
  foreach grammar [.main getGrammars] {
    set grammarRefNode [$document createElement "grammarRef"]
    if {[$grammar cget -name] == "<none>"} {
      continue
    }
    $grammarsNode appendChild $grammarRefNode
    $grammarRefNode setAttribute "name" [$grammar cget -name] 
    $grammarRefNode setAttribute "fileName" [$grammar getZippedFileName]
  }

  # save all machines refs
  set machinesNode [$document createElement "machines"]
  $globalNode appendChild $machinesNode
  foreach machine [.main getMachines] {
    set machineRefNode [$document createElement "machineRef"]
    if {[$machine cget -name] == "<none>"} {
      continue
    }
    $machinesNode appendChild $machineRefNode
    $machineRefNode setAttribute "name" [$machine cget -name]
    $machineRefNode setAttribute "fileName" [$machine getZippedFileName]
  }

  # save all experiment refs
  set experimentsNode [$document createElement "experiments"]
  $globalNode appendChild $experimentsNode
  foreach experiment [.main getExperiments] {
    set experimentRefNode [$document createElement "experimentRef"]
    if {[$experiment cget -name] == "<none>"} {
      continue
    }
    $experimentsNode appendChild $experimentRefNode
    $experimentRefNode setAttribute "name" [$experiment cget -name] 
    $experimentRefNode setAttribute "fileName" [$experiment getZippedFileName]
  }

  return $document
}

## ----------------------------------------------------------------------------
## initialize this object from a dom document
## @param document a dom document containing a /yada/gobal
## ----------------------------------------------------------------------------
body YadaPreferences::fromDom {document} {
  set rootNode [$document documentElement]
  set globalNode [$rootNode selectNodes "/yada/global"]
  fromDomNode $globalNode
}

## ----------------------------------------------------------------------------
## initialize this object from a dom node
## @param globalNode node containing the global attributes
## ----------------------------------------------------------------------------
body YadaPreferences::fromDomNode {globalNode} {
  configure \
    -significance [$globalNode getAttribute significance] \
    -timeTolerance [$globalNode getAttribute timeTolerance] \
    -editor [$globalNode getAttribute editor] \
    -weirdness [$globalNode getAttribute weirdness]
}

## ----------------------------------------------------------------------------
## Update toolbar, menubar and titlebar
## ----------------------------------------------------------------------------
body YadaPreferences::updateMenus {} {

  # set the state of the "Load" action
  set loadButton [[[.main getDocument Configure] component toolbar] component loadButton]
  if { [hasSavedState] } {
    $itk_component(fileMenu) entryconfigure 0 -state normal
    $loadButton configure -state normal
  } else {
    $itk_component(fileMenu) entryconfigure 0 -state disabled
    $loadButton configure -state disabled
  }

  # set the state of the "Save" action
  set saveButton [[[.main getDocument Configure] component toolbar] component saveButton]
  if {[isModified]} {
    $itk_component(fileMenu) entryconfigure 1 -state normal   
    $saveButton configure -state normal
  } else {
    $itk_component(fileMenu) entryconfigure 1 -state disabled
    $saveButton configure -state disabled
  }
}

## ----------------------------------------------------------------------------
## get the filename where the persistent state is saved in
## ----------------------------------------------------------------------------
body YadaPreferences::getFileName {} {
  global env
  return [getSecureFileName [file join $env(YADA_ETC) yada.xml]]
}

## ----------------------------------------------------------------------------
## called whenever this document gets activated
## ----------------------------------------------------------------------------
body YadaPreferences::activationHandle {} {
  chain
  ## activate custom file menu
  set fileButton [.main component fileButton]
  $fileButton configure -menu $itk_component(fileMenu)
}


