# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaJudgeBar - draw a chart of categories.
## This widget is used in the YadaDifference document to visualize the categories
## between performance and accuracy.
##
## \author Michael Daum
##
## $Id: YadaJudgeBar.tcl,v 1.11 2005-12-01 16:23:27 micha Exp $
## ----------------------------------------------------------------------------
class YadaJudgeBar {
  inherit itk::Widget


  # variables ----------------------------------------------------------------
  public variable betterAndFasterBackground "green1"
  public variable betterAndFasterForeground "black"
  public variable betterBackground "green2"
  public variable betterButSlowerBackground "green3"
  public variable betterButSlowerForeground "black"
  public variable betterForeground "black"
  public variable fasterBackground "lightblue"
  public variable fasterForeground "black"
  public variable harderBackground "deepskyblue3"
  public variable harderForeground "black"
  public variable sameBackground "gray"
  public variable sameForeground "black"
  public variable slowerBackground "yellow"
  public variable slowerForeground "black"
  public variable softerBackground "deepskyblue1"
  public variable softerForeground "black"
  public variable weirdBackground "pink"
  public variable weirdForeground "black"
  public variable worseAndSlowerBackground "red3"
  public variable worseAndSlowerForeground "white"
  public variable worseBackground "red1"
  public variable worseButFasterBackground "orange"
  public variable worseButFasterForeground "black"
  public variable worseForeground "white"

  constructor {args} {}; ## \type TclList

  private variable _totalWordgraphs 0
  private variable _fasterWordgraphs 0
  private variable _betterWordgraphs 0
  private variable _betterAndFasterWordgraphs 0
  private variable _betterButSlowerWordgraphs 0
  private variable _softerWordgraphs 0
  private variable _harderWordgraphs 0
  private variable _sameWordgraphs 0
  private variable _worseButFasterWordgraphs 0
  private variable _worseAndSlowerWordgraphs 0
  private variable _worseWordgraphs 0
  private variable _slowerWordgraphs 0
  private variable _weirdWordgraphs 0

  # methods ------------------------------------------------------------------
  public method draw {args}; ## \type TclList
  private method _judgeEnter {x y}; ## \type TclNumber, TclNumber
  private method _judgeLeave {}

};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaJudgeBar::constructor {args} {

  itk_component add canvas {
    canvas $itk_interior.canvas \
      -height 15 \
      -borderwidth 2 \
      -relief sunk
  } {
    usual
    keep -height -width -relief -borderwidth
  }

  eval itk_initialize $args

  set cv $itk_component(canvas)
  $cv create rectangle -100 -100 -100 -100 -tag betterRec -fill $betterBackground
  $cv create rectangle -100 -100 -100 -100 -tag betterAndFasterRec -fill $betterAndFasterBackground
  $cv create rectangle -100 -100 -100 -100 -tag betterButSlowerRec -fill $betterButSlowerBackground
  $cv create rectangle -100 -100 -100 -100 -tag fasterRec -fill $fasterBackground
  $cv create rectangle -100 -100 -100 -100 -tag softerRec -fill $softerBackground
  $cv create rectangle -100 -100 -100 -100 -tag sameRec -fill $sameBackground
  $cv create rectangle -100 -100 -100 -100 -tag harderRec -fill $harderBackground
  $cv create rectangle -100 -100 -100 -100 -tag slowerRec -fill $slowerBackground
  $cv create rectangle -100 -100 -100 -100 -tag worseButFasterRec -fill $worseButFasterBackground
  $cv create rectangle -100 -100 -100 -100 -tag worseAndSlowerRec -fill $worseAndSlowerBackground
  $cv create rectangle -100 -100 -100 -100 -tag worseRec -fill $worseBackground
  $cv create rectangle -100 -100 -100 -100 -tag weirdRec -fill $weirdBackground
  $cv create text -100 -100 -tag betterText -fill $betterForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag betterAndFasterText -fill $betterAndFasterForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag betterButSlowerText -fill $betterButSlowerForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag fasterText -fill $fasterForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag softerText -fill $softerForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag sameText -fill $sameForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag harderText -fill $harderForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag slowerText -fill $slowerForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag worseButFasterText -fill $worseButFasterForeground  \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*} -text "" -anchor c
  $cv create text -100 -100 -tag worseAndSlowerText -fill $worseAndSlowerForeground \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*} -text "" -anchor c
  $cv create text -100 -100 -tag worseText -fill $worseForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}
  $cv create text -100 -100 -tag weirdText -fill $weirdForeground -text "" -anchor c \
    -font {-adobe-helvetica-medium-r-normal--10-*-*-*-*-*-*-*}

  # canvas bindings
  $cv bind all <Any-Enter> [code $this _judgeEnter %X %Y]
  $cv bind all <Any-Leave> [code $this _judgeLeave]

  # bindings
  bind $cv <Configure> [code $this draw]

  # packing
  pack $itk_component(canvas) -fill both -expand 1
}

## ----------------------------------------------------------------------------
## draw
## ----------------------------------------------------------------------------
body YadaJudgeBar::draw {args} {

  set noArgs [llength $args]

  if {$noArgs == 12} {
    set pos 0
    set _totalWordgraphs 0
    foreach argument {
      _fasterWordgraphs _betterWordgraphs
      _betterAndFasterWordgraphs _betterButSlowerWordgraphs _softerWordgraphs
      _harderWordgraphs _sameWordgraphs _worseButFasterWordgraphs
      _worseAndSlowerWordgraphs _worseWordgraphs _slowerWordgraphs
      _weirdWordgraphs
    } {
      set $argument [lindex $args $pos]
      incr pos
      incr _totalWordgraphs [subst \$$argument]
    }
  } elseif {$noArgs != 0} {
    error "ERROR: illegal number of parameters to YadaJudgeBar::draw"
  }

  set cv $itk_component(canvas)
  set cvHeight [winfo height $cv]
  set cvWidth [winfo width $cv]

  set lastX 0
  foreach category {betterAndFaster better betterButSlower \
		 faster softer same weird harder slower \
		 worseButFaster worse worseAndSlower} {

    set wordgraphs [subst \$_${category}Wordgraphs]
    if {$wordgraphs > 0} {
      if {$_totalWordgraphs > 0} {
	set percentage [expr $wordgraphs / ($_totalWordgraphs + 0.0) ]
      } else {
	set percentage 0.0
      }
      set newX [expr $lastX + $percentage * $cvWidth]
      $cv coords ${category}Rec $lastX 0 $newX $cvHeight 

      if {$percentage >= 0.02} {
	$cv coords ${category}Text [expr $lastX + ($newX - $lastX) / 2.0] [expr $cvHeight / 2.0]
	$cv itemconfigure ${category}Text -text [format "%.f%%" [expr $percentage * 100]]
      } else {
	$cv coords ${category}Text -100 -100
      }

      set lastX $newX
    } else {
      $cv coords ${category}Rec -100 -100 -100 -100
      $cv coords ${category}Text -100 -100
    }
  }

}

## ----------------------------------------------------------------------------
## _judgeEnter
## ----------------------------------------------------------------------------
body YadaJudgeBar::_judgeEnter {x y} {
  set cv $itk_component(canvas)
  set id [$cv find withtag current]
  set tag [lindex [$cv itemcget $id -tags] 0]

  regexp "\(.*\)\(Rec\|Text\)" $tag dummy category


  set noWordgraphs [subst \$_${category}Wordgraphs]
  set percentage [expr $noWordgraphs / ($_totalWordgraphs + 0.0) * 100]

  .balloon on $x $y [format "%s - %d (%.2f%%)" $category $noWordgraphs $percentage] 
}

## ----------------------------------------------------------------------------
## _judgeLeave
## ----------------------------------------------------------------------------
body YadaJudgeBar::_judgeLeave {} {
  .balloon off
}
