# Copyright (C) 1997-2005 The CDG Team <cdg@nats.informatik.uni-hamburg.de>
#  
# This file is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. For
# more details read COPYING in the root of this distribution.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY, to the extent permitted by law; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

## ----------------------------------------------------------------------------
## YadaMachines - configure document for all machines.
## \ingroup YadaConfiguration
##
## \author Michael Daum
##
## $Id: YadaMachines.tcl,v 1.16 2006-06-07 17:38:54 yvonne Exp $
## ----------------------------------------------------------------------------
class YadaMachines {
  inherit YadaConfigDocument

  # variables ----------------------------------------------------------------
  private variable _noNodes 0
  private variable _noSelNodes 0
  private variable _nodeOfRow
  private variable _rowOfNode
  
  # methods ------------------------------------------------------------------
  public method init {}
  public method updateCombos {}
  public method changedConnection {node}
  constructor {args} {}; ## \type TclList

  private method _commit {}
  private method _defaults {}
  private method _new {}
  private method _delete {}
  private method _select {{machineName ""}}; ## \type TclString
  private method _leaveTableCommand {}
  private method _escapeCommand {}
  private method _returnCommand {}
  private method _getMarkedNodes {}
  private method _mark {aspect}; ## \type TclString
  private method _isMarked {row}; ## \type Integer
  private method _invertNodes {}
  private method _selectNodes {}
  private method _unselectNodes {}
  private method _addNode {}
  private method _deleteNodes {}
  private method _connectNodes {}
  private method _getSelectedRows {}
  private method _getSelectedNodes {}
};

## ----------------------------------------------------------------------------
## constructor
## ----------------------------------------------------------------------------
body YadaMachines::constructor {args} {

  set _allAspects {name nodes}

  # edit menu
  set index [$itk_component(editMenu) index "Delete"]
  incr index
  $itk_component(editMenu) insert [incr index] command \
       	-label "Select" \
       	-underline 0 \
       	-command [code $this _selectNodes]
  $itk_component(editMenu) insert [incr index] command \
       	-label "Unselect" \
       	-underline 0 \
       	-command [code $this _unselectNodes]
  $itk_component(editMenu) insert [incr index] command \
       	-label "Invert Selection" \
       	-underline 0 \
       	-command [code $this _invertNodes]
  $itk_component(editMenu) insert [incr index] separator

  $itk_component(editMenu) insert [incr index] command \
       	-label "Add Node" \
       	-underline 0 \
       	-command [code $this _addNode]
  $itk_component(editMenu) insert [incr index] command \
       	-label "Delete Nodes" \
       	-underline 0 \
       	-command [code $this _deleteNodes]
  $itk_component(editMenu) insert [incr index] command \
       	-label "Connect Nodes" \
       	-underline 0 \
       	-command [code $this _connectNodes]
  $itk_component(editMenu) insert [incr index] separator

  # more buttons
  itk_component add selButton {
    button $itk_component(topButtonFrame).selButton \
      -text "Select" \
      -command [code $this _selectNodes]
  }

  itk_component add unselButton {
    button $itk_component(topButtonFrame).unselButton \
      -text "Unselect" \
      -command [code $this _unselectNodes]
  }

  itk_component add invertButton {
    button $itk_component(topButtonFrame).invertButton \
      -text "Invert" \
      -command [code $this _invertNodes]
  }

  itk_component add addButton {
    button $itk_component(topButtonFrame).addButton \
      -text "Add Node" \
      -command [code $this _addNode]
  }

  itk_component add delButton {
    button $itk_component(topButtonFrame).delButton \
      -text "Del Node" \
      -command [code $this _deleteNodes]
  }

  itk_component add conButton {
    button $itk_component(topButtonFrame).conButton \
      -text "Connect" \
      -command [code $this _connectNodes]
  }


  # counter
  itk_component add counterFrame {
    frame $itk_component(childsite).counterFrame 
  } {}
  itk_component add totalCount {
    iwidgets::entryfield $itk_component(counterFrame).totalCount \
      -labeltext "Total:"\
      -labelpos w \
      -state disabled \
      -textvariable [scope _noNodes] \
      -borderwidth 2 \
      -disabledforeground black \
  } {}

  itk_component add selCount {
    iwidgets::entryfield $itk_component(counterFrame).selCount \
      -labeltext "Selected:"\
      -labelpos w \
      -state disabled \
      -textvariable [scope _noSelNodes] \
      -borderwidth 2 \
      -disabledforeground black \
  } {}

  # nodes table
  itk_component add nodesMark {
    frame $itk_component(childsite).nodesMark \
      -borderwidth 2
  } {}

  itk_component add nodesLabel {
    label $itk_component(nodesMark).label \
      -text "Nodes:" \
      -anchor nw
  } {}
  itk_component add table {
    MyTable $itk_component(nodesMark).table \
      -exportselection 1 \
      -selecttitles 1 \
      -rowseparator "\n" \
      -colseparator "\t" \
      -rows 20 \
      -cols 5 \
      -state normal \
      -cellanchor w \
      -colstretchmode last \
      -rowstretchmode none \
      -selectmode extended \
      -selecttype row \
      -titlerows 1 \
      -titlecols 1 \
      -borderwidth 3 \
      -outerborderwidth 2 \
      -outerrelief sunk \
      -selforeground "" \
      -selbackground "" \
      -font {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} \
      -hscrollmode static \
      -vscrollmode static \
  } {}

  $itk_component(table) setCell \
      0,0 "no." \
      0,1 "host" \
      0,2 "access" \
      0,3 "cpus" \
      0,4 "connection" \

  $itk_component(table) tag config title \
    -font {-adobe-helvetica-medium-r-normal--12-*-*-*-*-*-*-*} -anchor c 
  $itk_component(table) tag config sel -fg "" -bg "" -relief raised -bd 1
  $itk_component(table) tag config mark -bg green3 -fg black

  $itk_component(table) width 0 4 1 30 2 30 3 10 4 20

  grid $itk_component(nodesMark) -sticky news -row 1 -column 0 -pady 10 \
    -columnspan 2
  pack $itk_component(nodesLabel) -fill x -side top
  pack $itk_component(table) -fill both -expand 1 -side top
  
  grid $itk_component(counterFrame) -sticky ew -row 2 -column 0 -pady 10 \
    -columnspan 2
  pack $itk_component(totalCount) \
       $itk_component(selCount) -side left

  grid rowconfigure $itk_component(childsite) 1 -weight 1

  pack $itk_component(selButton) \
       $itk_component(unselButton) \
       $itk_component(invertButton) \
       $itk_component(addButton) \
       $itk_component(delButton) \
       $itk_component(conButton) \
       -side top -fill x -pady 5

  # bindings
  bind [$itk_component(table) component table] <Double-Button-1> \
    +[code $this _invertNodes]

  #set hull $itk_component(hull)
  set tableInTable [$itk_component(table) component table]
  bind $tableInTable <Leave> [code $this _leaveTableCommand]
  bind $tableInTable <FocusOut> [code $this _leaveTableCommand]
  bind $tableInTable <KeyPress-Escape> [code "$this _escapeCommand; break"]
  bind $tableInTable <KeyPress-Return> [code "$this _returnCommand; break"]

  eval itk_initialize $args
}

## ----------------------------------------------------------------------------
## init
## ----------------------------------------------------------------------------
body YadaMachines::init {} {
  if {$_isInitialized} {
    return
  }

  # get xml data
  set found 0
  set prefs [[.main component Configure] component prefs]
  foreach itemRefNode [$prefs evalXpath "/yada/global/$name/*"] {
    set itemName [$itemRefNode getAttribute "name"]
    set itemFileName [$itemRefNode getAttribute "fileName"]
    if {$itemName == "<none>"} {
      continue
    }
    if {[file exists $itemFileName]} {
      set item [newMachine -name $itemName]
      $item load $itemFileName
      if {[addItem $item]} {
	incr found
      } else {
	itcl::delete object $item
      }
    }
  }

  if {!$found} {
    _defaults
  }

  # required localhost machine
  set noneMachine [newMachine -name "<none>"]
  addItem $noneMachine
  $noneMachine newNode -host "<none>" -access "<none>" -cpus 0 -isActive 0

  if {[getItem "localhost"] == ""} {
    set localhost [newMachine -name "localhost"]
    $localhost newNode -host "localhost" -access "local" -cpus 1
    addItem $localhost
  }
  
  chain
  _selectFirst
}

## ----------------------------------------------------------------------------
## _select
## ----------------------------------------------------------------------------
body YadaMachines::_select {{machineName ""}} {

  # get a machine name from the combobox if we didn't provide one
  if {$machineName == ""} {
    set machineName [$itk_component(name) get]
  } 

  set currentItem [getItem $machineName]
  if {$currentItem == ""} {
    printMessage "ERROR: machine `$machineName' is unknown" error
    return
  }

  # check if we didn't commit changes
  if {![_askChange]} {
    return
  }

  # set the active machine
  set prevItem [getCurrentItem]
  setCurrentItem $currentItem

  # update title bar and menus
  updateWindowTitle
  updateMenus

  # update the name box
  set box $itk_component(name)
  set values [[$box component list] get 0 end]
  set value [$currentItem cget -name]
  set index [lsearch $values $value]
  if {$index > 0} {
    $box selection set $index $index
  }
  $box delete entry 0 end
  $box insert entry 0 $value

  # show the machine

  # clear the tables
  set table $itk_component(table)
  set rmax [$table cget -rows]
  $table erase 1,0 $rmax,5
  $table clear tags 1,0 $rmax,5
  $table selection clear all

  set currentItem [getCurrentItem]
  set nodes [$currentItem getNodes]
  set _noNodes [llength $nodes]
  set _noSelNodes 0
  array unset _nodeOfRow
  array unset _rowOfNode

  # resize the table
  if {$_noNodes < 100} {
    set rmax 100
  } else {
    set rmax [expr $_noNodes +1]
  }
  $table configure -rows $rmax

  set row 1
  foreach node $nodes {
    set _nodeOfRow($row) $node
    set _rowOfNode($node) $row

    set connected ""
    if {[$node cget -connection] == 1} {
      set connected connected
    }
    
    $table setCell \
      $row,0 [format "%4d" $row ] \
      $row,1 [$node cget -host] \
      $row,2 [$node cget -access] \
      $row,3 [$node cget -cpus] \
      $row,4 $connected
    if {[$node cget -isActive]} {
      $table tag row mark $row
      incr _noSelNodes
    } else {
      $table tag row "" $row
    }

    # configure Callbacks for the column "connnection" in the table
    # TODO _select is called a lot of times. Configuration of nodes should be elsewhere
    $node configure -connectionCallback [code $this changedConnection]

    incr row
  }

  # adjust the view
  $table see 1,1


  # remove marking
  _unmark
}

## ----------------------------------------------------------------------------
## create YadaMachine objects sourcing a tcl resource file.
## The resource file is either @c YADA_ETC/machines.tcl if it exists or
## @c YADA_LIB/machines.tcl.sample. The former allows the user to programm
## machines. 
## ----------------------------------------------------------------------------
body YadaMachines::_defaults {} {
  global env

  # get default resources
  set fileName [file join $::env(YADA_ETC) machines.tcl]
  if {[file exists $fileName]} {
    .main printStatus "... loading $name file '$fileName'."
  } else {
    .main printStatus "... configure file '$fileName' not found ... using defaults."
    set fileName [file join $env(YADA_LIB) machines.tcl.sample]
  }

  # ... configured machines
  if {[catch {source $fileName} errMsg]} {
    bgerror "There's an error in your machines init file $fileName!"
  }

  # assert machines into the class
  set n 0
  foreach machine [find objects -class YadaMachine] {
    if {[addItem $machine]} {
      incr n
    } else {
      itcl::delete object $machine
    }
  }

  .main printStatus "... built $n machine(s)"
}

## ----------------------------------------------------------------------------
## _commit
## ----------------------------------------------------------------------------
body YadaMachines::_commit {} {

  set currentItem [getCurrentItem]
  set oldName [$currentItem cget -name]

  if {![chain]} {
    return
  }

  # adjust each experiment using a machine with a changed name
  set newName [$currentItem cget -name]
  if {$oldName != $newName} {
    foreach experiment [.main getExperiments] {
      if {[$experiment cget -machineName] == $oldName} {
	$experiment configure -machineName $newName
      }
    }
  }

  # Mark as modified
  $currentItem configure -isModified 1

  # store the table into the YadaNode objects
  set table $itk_component(table)
  for {set row 1} {$row <= $_noNodes} {incr row} {
    set host [string trim [$table getCell $row,1]]

    if {[info exists _nodeOfRow($row)]} {
      set node $_nodeOfRow($row)
      set seen($node) 1
    } else {
      if {$host != ""} {
	set node [$currentItem newNode]
	set _nodeOfRow($row) $node
	set _rowOfNode($node) $row
	set seen($node) 1
      }
    }

    if {$host != ""} {
      $node configure \
	-host $host \
	-access [string trim [$table getCell $row,2]] \
	-cpus [string trim [$table getCell $row,3]] \
	-isActive [_isMarked $row]
    } 
  }

  # delete unseen nodes
  foreach node [$currentItem getNodes] {
    if {[info exists seen($node)]} {
      continue
    }
    itcl::delete object $node
  }

  # remove marking
  _unmark
  _select
}

## ----------------------------------------------------------------------------
## delete a YadaMachine
## ----------------------------------------------------------------------------
body YadaMachines::_delete {} {

  # get the machine name
  set currentItem [getCurrentItem]
  set machineName [$currentItem cget -name]

  # machine "<none>" is undeleteable
  if {$machineName == "<none>" || $machineName == "localhost"} {
    return
  }

  # set experiments using the deleted machine to machine <none>
  set noExperiments 0
  foreach experiment [.main getExperiments] {
    if {[$experiment cget -machineName] == $machineName} {
      incr noExperiments
    }
  }

  set msgText "Are you sure that you want to delete machine '$machineName'?"
  if {$noExperiments > 0} {
    if {$noExperiments == 1} {
      append msgText "\nThere is one experiment using it."
    } else {
      append msgText "\nThere are $noExperiments experiments using it."
    }
  }

  # are you sure?
  set msg [iwidgets::messagedialog .msg \
    -text $msgText \
    -title "Yada - Caution"  \
    -bitmap questhead \
    -modality application \
    -master $itk_interior]
  $msg buttonconfigure OK -text "Yes"
  $msg buttonconfigure Cancel -text "No"
  $msg center .
  set result [$msg activate]
  destroy $msg
  if {!$result} {
    return
  }

  # delete the data
  removeItem $currentItem
  $currentItem delete
  itcl::delete object $currentItem

  # update the names
  updateCombos

  # set experiments using the deleted machine to machine <none>
  foreach experiment [.main getExperiments] {
    if {[$experiment cget -machineName] == $machineName} {
      $experiment configure -machineName "<none>" -isModified 1
    }
  }

  # show something
  _selectFirst
}

## ----------------------------------------------------------------------------
## create a new machine.
## This is done by cloning the current one. We only ask for a new name.
## ----------------------------------------------------------------------------
body YadaMachines::_new {} {

  # ask for a new name
  set prd [iwidgets::promptdialog .prd \
    -title "Yada - New machine" \
    -modality application \
    -labeltext "Name:" \
    -labelpos w]

  set machineName [$itk_component(name) get]
  set entry [$prd component prompt]
  $entry configure -width 40
  $entry insert 0 $machineName

  $prd center .
  $prd hide Apply
  $prd hide Help
  after idle "focus [$prd component prompt]"

  set result [$prd activate]
  set machineName [$prd get]
  destroy $prd
  
  if {!$result} {
    return
  }

  set item [getItem $machineName]
  if {$item != ""} {
    printMessage "ERROR: machine '$machineName' exists already." error
    return [_new]
  }

  # build a new machine
  set currentItem [getCurrentItem]
  setCurrentItem [$currentItem clone -name $machineName]
  
  # display it 
  _select $machineName
}

## ----------------------------------------------------------------------------
## update the list of known aspects
## ----------------------------------------------------------------------------
body YadaMachines::updateCombos {} {
  chain
}

## ----------------------------------------------------------------------------
## _leaveTableCommand
## ----------------------------------------------------------------------------
body YadaMachines::_leaveTableCommand {} {
  set table $itk_component(table)
  set lastActiveCell [$table tag cell active]
  if {$lastActiveCell == ""} {
    return
  }

  scan $lastActiveCell "%d,%d" row col
  $table activate -1,-1
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## _returnCommand
## ----------------------------------------------------------------------------
body YadaMachines::_returnCommand {} {
  set table $itk_component(table)
  set lastActiveCell [$table tag cell active]
  scan $lastActiveCell "%d,%d" row col
  set colMax [expr [$table cget -cols] -1]
  if {$col == $colMax} {
    incr row
    set col 1
  } else {
    incr col
  }

  $table activate $row,$col
  $table see $row,$col
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## _escapeCommand
## ----------------------------------------------------------------------------
body YadaMachines::_escapeCommand {} {
  set table $itk_component(table)
  set lastActiveCell [$table tag cell active]
  scan $lastActiveCell "%d,%d" row col
  
  if {$col == 0} {
    $table selection clear all
  } else {
    $table activate -1,-1
  }
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## get a list of selected rows
## ----------------------------------------------------------------------------
body YadaMachines::_getSelectedRows {} {
  set rows {}

  set table $itk_component(table)
  for {set row 1} {$row <= $_noNodes} {incr row} {
    if {[$table selection includes $row,1]} {
      lappend rows $row
    }
  }
  return $rows
}

## ----------------------------------------------------------------------------
## get a list of selected nodes
## ----------------------------------------------------------------------------
body YadaMachines::_getSelectedNodes {} {
  set nodes {}

  foreach row [_getSelectedRows] {
    if {[info exists _nodeOfRow($row)]} {
      set node $_nodeOfRow($row)
      lappend nodes $node
    }
  }

  return $nodes
}

## ----------------------------------------------------------------------------
## invert the activation of marked nodes
## ----------------------------------------------------------------------------
body YadaMachines::_invertNodes {} {

  set table $itk_component(table)
  foreach row [_getSelectedRows] {
    if {[_isMarked $row]} {
      $table tag row "" $row
    } else {
      $table tag row mark $row
    }
  }
  
  set _noSelNodes [llength [_getMarkedNodes]]
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## activate selected nodes
## ----------------------------------------------------------------------------
body YadaMachines::_selectNodes {} {

  set table $itk_component(table)
  foreach row [_getSelectedRows] {
    $table tag row mark $row
  }
  
  set _noSelNodes [llength [_getMarkedNodes]]
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## deactivate selected nodes
## ----------------------------------------------------------------------------
body YadaMachines::_unselectNodes {} {

  set table $itk_component(table)
  foreach row [_getSelectedRows] {
    $table tag row "" $row
  }
  
  set _noSelNodes [llength [_getMarkedNodes]]
  _mark "nodes"
  
}

## ----------------------------------------------------------------------------
## get a list of marked nodes
## ----------------------------------------------------------------------------
body YadaMachines::_getMarkedNodes {} {
  set result {}
  set table $itk_component(table)
  for {set row 1} {$row <= $_noNodes} {incr row} {
    if {[_isMarked $row]} {
      lappend result $_nodeOfRow($row)
    }
  }

  return $result
}

## ----------------------------------------------------------------------------
## returns true if the given row is marked in the table
## ----------------------------------------------------------------------------
body YadaMachines::_isMarked {row} {
  return [$itk_component(table) tag includes mark $row,1]
}

## ----------------------------------------------------------------------------
## mark the view as changed if so
## ----------------------------------------------------------------------------
body YadaMachines::_mark {aspect} {

  if {$aspect != "nodes"} {
    chain
    return 
  }

  set oldData {}
  set newData {}
  set currentItem [getCurrentItem]

  # get a string representation of the active nodes' information
  foreach node [$currentItem getNodes] {
    lappend oldData \
      [list \
	[$node cget -host] \
	[$node cget -access] \
	[$node cget -cpus] \
	[$node cget -isActive]]
  }
  set oldData [lsort $oldData]
  
  # get a string representation of the marked nodes' information
  set table $itk_component(table)
  for {set row 1} {$row <= $_noNodes} {incr row} {
    lappend newData \
      [list \
	[string trim [$table getCell $row,1]] \
	[string trim [$table getCell $row,2]] \
	[string trim [$table getCell $row,3]] \
	[_isMarked $row]]
  }
  set newData [lsort $newData]

  chain $aspect $newData $oldData
}

## ----------------------------------------------------------------------------
## add a node to the current machine.
## This will copy the last node's specs or use the first one that has been 
## selected.
## ----------------------------------------------------------------------------
body YadaMachines::_addNode {} {
  set selectedRows [_getSelectedRows]
  set srcRow $_noNodes
  set dstRow $_noNodes
  incr dstRow
  if {$selectedRows != {}} {
    set srcRow [lindex $selectedRows 0]
  }

  set table $itk_component(table)

  $table setCell \
    $dstRow,0 [format "%4d" $dstRow ] \
    $dstRow,1 [$table getCell $srcRow,1] \
    $dstRow,2 [$table getCell $srcRow,2] \
    $dstRow,3 [$table getCell $srcRow,3]

  incr _noNodes

  $table see $dstRow,0
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## remove the selected nodes from the current machine.
## ----------------------------------------------------------------------------
body YadaMachines::_deleteNodes {} {
  if {$_noNodes == 1} {
    printMessage "ERROR: cannot delete last node of this machine." error
    return
  }
  set table $itk_component(table)
  foreach row [_getSelectedRows] {
    $table erase $row,1 $row,5
    $table clear tags $row,0 $row,5
    if {[info exists _nodeOfRow($row)]} {
      set node $_nodeOfRow($row)
      unset _rowOfNode($node)
      unset _nodeOfRow($row)
    }
  }
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## connect the selected nodes to a cdgp server
## ----------------------------------------------------------------------------
body YadaMachines::_connectNodes {} {
  set table $itk_component(table)
  foreach node [_getSelectedNodes] {
    set row $_rowOfNode($node)
watch "row $_rowOfNode($node)"
    
    # connect node
    set pid [$node connect]
      
    # write connection status in table
    if {$pid == 0} {
      $table setCell $row,4 "connection failed"
    } else {
      # if Node is not active, activate it
      if {![$node cget -isActive]} {
        $node configure -isActive 1
        $table tag row mark $row
        set _noSelNodes [llength [_getMarkedNodes]]
      }
      $table setCell $row,4 "connected"
      $node configure -connectionCallback [code $this changedConnection] \
	  -deactivationCallback [code $this deactivateNode]
    }
  }  
  _mark "nodes"
}

## ----------------------------------------------------------------------------
## write connection status in table
## ----------------------------------------------------------------------------
body YadaMachines::changedConnection {node} {
  set table $itk_component(table)
watch "row $_rowOfNode($node)"
  set row $_rowOfNode($node)
  set connection [$node cget -connection]

  if {$connection == 0} {
    $table setCell $row,4 "lost connection"
  } elseif {$connection == 1} {
    $table setCell $row,4 "connected"
  } elseif {$connection == -1} {

    $table tag row "" $row
    $table setCell $row,4 "connection failed"
  
    set _noSelNodes [llength [_getMarkedNodes]]
    _mark "nodes"
  }
}
